#include "shaders/gl_shaders.hpp"

#include "base/assert.hpp"

#include "std/target_os.hpp"

#include <array>

namespace gpu
{
char const * GL3_SHADER_VERSION = "#version 150 core \n";
char const * GLES3_SHADER_VERSION = "#version 300 es \n";

char const POSITION_ACCURACY3D_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec2 a_normal; \n\
  attribute vec2 a_colorTexCoords; \n\
  uniform vec3 u_position; \n\
  uniform float u_accuracy; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_zScale; \n\
  varying vec2 v_colorTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 position = vec4(u_position.xy, 0.0, 1.0) * u_modelView; \n\
    vec4 normal = vec4(a_normal * u_accuracy, 0.0, 0.0); \n\
    position = (position + normal) * u_projection; \n\
    gl_Position = applyPivotTransform(position, u_pivotTransform, u_position.z * u_zScale); \n\
    v_colorTexCoords = a_colorTexCoords; \n\
  } \n\
";

char const GLES3_POSITION_ACCURACY3D_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec2 a_normal; \n\
  in vec2 a_colorTexCoords; \n\
  uniform vec3 u_position; \n\
  uniform float u_accuracy; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_zScale; \n\
  out vec2 v_colorTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 position = vec4(u_position.xy, 0.0, 1.0) * u_modelView; \n\
    vec4 normal = vec4(a_normal * u_accuracy, 0.0, 0.0); \n\
    position = (position + normal) * u_projection; \n\
    gl_Position = applyPivotTransform(position, u_pivotTransform, u_position.z * u_zScale); \n\
    v_colorTexCoords = a_colorTexCoords; \n\
  } \n\
";

char const AREA3D_OUTLINE_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec3 a_position; \n\
  attribute vec2 a_colorTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_zScale; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  varying LOW_P vec4 v_color; \n\
  #else \n\
  varying vec2 v_colorTexCoords; \n\
  #endif \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pos = vec4(a_position, 1.0) * u_modelView; \n\
    pos.xyw = (pos * u_projection).xyw; \n\
    pos.z = a_position.z * u_zScale; \n\
    gl_Position = u_pivotTransform * pos; \n\
  #ifdef VULKAN \n\
    gl_Position.y = -gl_Position.y; \n\
    gl_Position.z = (gl_Position.z  + gl_Position.w) * 0.5; \n\
  #endif \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture2D(u_colorTex, a_colorTexCoords); \n\
  #else \n\
    v_colorTexCoords = a_colorTexCoords; \n\
  #endif \n\
  } \n\
";

char const GLES3_AREA3D_OUTLINE_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 a_position; \n\
  in vec2 a_colorTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_zScale; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  out LOW_P vec4 v_color; \n\
  #else \n\
  out vec2 v_colorTexCoords; \n\
  #endif \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pos = vec4(a_position, 1.0) * u_modelView; \n\
    pos.xyw = (pos * u_projection).xyw; \n\
    pos.z = a_position.z * u_zScale; \n\
    gl_Position = u_pivotTransform * pos; \n\
  #ifdef VULKAN \n\
    gl_Position.y = -gl_Position.y; \n\
    gl_Position.z = (gl_Position.z  + gl_Position.w) * 0.5; \n\
  #endif \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture(u_colorTex, a_colorTexCoords); \n\
  #else \n\
    v_colorTexCoords = a_colorTexCoords; \n\
  #endif \n\
  } \n\
";

char const ROUTE_ARROW_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec4 a_position; \n\
  attribute vec2 a_normal; \n\
  attribute vec2 a_colorTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_arrowHalfWidth; \n\
  varying vec2 v_colorTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec2 transformedAxisPos = (vec4(a_position.xy, 0.0, 1.0) * u_modelView).xy; \n\
    if (dot(a_normal, a_normal) != 0.0) \n\
    { \n\
      vec2 norm = a_normal * u_arrowHalfWidth; \n\
      transformedAxisPos = calcLineTransformedAxisPos(transformedAxisPos, a_position.xy + norm, \n\
                                                      u_modelView, length(norm)); \n\
    } \n\
    v_colorTexCoords = a_colorTexCoords; \n\
    vec4 pos = vec4(transformedAxisPos, a_position.z, 1.0) * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
  } \n\
";

char const GLES3_ROUTE_ARROW_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec4 a_position; \n\
  in vec2 a_normal; \n\
  in vec2 a_colorTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_arrowHalfWidth; \n\
  out vec2 v_colorTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec2 transformedAxisPos = (vec4(a_position.xy, 0.0, 1.0) * u_modelView).xy; \n\
    if (dot(a_normal, a_normal) != 0.0) \n\
    { \n\
      vec2 norm = a_normal * u_arrowHalfWidth; \n\
      transformedAxisPos = calcLineTransformedAxisPos(transformedAxisPos, a_position.xy + norm, \n\
                                                      u_modelView, length(norm)); \n\
    } \n\
    v_colorTexCoords = a_colorTexCoords; \n\
    vec4 pos = vec4(transformedAxisPos, a_position.z, 1.0) * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
  } \n\
";

char const TEXT_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec2 a_colorTexCoord; \n\
  attribute vec2 a_maskTexCoord; \n\
  attribute vec4 a_position; \n\
  attribute vec2 a_normal; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  varying LOW_P vec4 v_color; \n\
  #else \n\
  varying vec2 v_colorTexCoord; \n\
  #endif \n\
  varying vec2 v_maskTexCoord; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pos = vec4(a_position.xyz, 1) * u_modelView; \n\
    vec4 shiftedPos = vec4(a_normal, 0.0, 0.0) + pos; \n\
    gl_Position = applyPivotTransform(shiftedPos * u_projection, u_pivotTransform, 0.0); \n\
   \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture2D(u_colorTex, a_colorTexCoord); \n\
  #else \n\
    v_colorTexCoord = a_colorTexCoord; \n\
  #endif \n\
    v_maskTexCoord = a_maskTexCoord; \n\
  } \n\
";

char const GLES3_TEXT_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec2 a_colorTexCoord; \n\
  in vec2 a_maskTexCoord; \n\
  in vec4 a_position; \n\
  in vec2 a_normal; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  out LOW_P vec4 v_color; \n\
  #else \n\
  out vec2 v_colorTexCoord; \n\
  #endif \n\
  out vec2 v_maskTexCoord; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pos = vec4(a_position.xyz, 1) * u_modelView; \n\
    vec4 shiftedPos = vec4(a_normal, 0.0, 0.0) + pos; \n\
    gl_Position = applyPivotTransform(shiftedPos * u_projection, u_pivotTransform, 0.0); \n\
   \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture(u_colorTex, a_colorTexCoord); \n\
  #else \n\
    v_colorTexCoord = a_colorTexCoord; \n\
  #endif \n\
    v_maskTexCoord = a_maskTexCoord; \n\
  } \n\
";

char const DASHED_LINE_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec3 a_position; \n\
  attribute vec3 a_normal; \n\
  attribute vec2 a_colorTexCoord; \n\
  attribute vec4 a_maskTexCoord; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  varying vec2 v_colorTexCoord; \n\
  varying vec2 v_maskTexCoord; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec2 normal = a_normal.xy; \n\
    float halfWidth = length(normal); \n\
    vec2 transformedAxisPos = (vec4(a_position.xy, 0.0, 1.0) * u_modelView).xy; \n\
    if (halfWidth != 0.0) \n\
    { \n\
      transformedAxisPos = calcLineTransformedAxisPos(transformedAxisPos, a_position.xy + normal, \n\
                                                      u_modelView, halfWidth); \n\
    } \n\
    float uOffset = min(length(vec4(kShapeCoordScalar, 0, 0, 0) * u_modelView) * a_maskTexCoord.x, 1.0); \n\
    v_colorTexCoord = a_colorTexCoord; \n\
    v_maskTexCoord = vec2(a_maskTexCoord.y + uOffset * a_maskTexCoord.z, a_maskTexCoord.w); \n\
    vec4 pos = vec4(transformedAxisPos, a_position.z, 1.0) * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
  } \n\
";

char const GLES3_DASHED_LINE_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 a_position; \n\
  in vec3 a_normal; \n\
  in vec2 a_colorTexCoord; \n\
  in vec4 a_maskTexCoord; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  out vec2 v_colorTexCoord; \n\
  out vec2 v_maskTexCoord; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec2 normal = a_normal.xy; \n\
    float halfWidth = length(normal); \n\
    vec2 transformedAxisPos = (vec4(a_position.xy, 0.0, 1.0) * u_modelView).xy; \n\
    if (halfWidth != 0.0) \n\
    { \n\
      transformedAxisPos = calcLineTransformedAxisPos(transformedAxisPos, a_position.xy + normal, \n\
                                                      u_modelView, halfWidth); \n\
    } \n\
    float uOffset = min(length(vec4(kShapeCoordScalar, 0, 0, 0) * u_modelView) * a_maskTexCoord.x, 1.0); \n\
    v_colorTexCoord = a_colorTexCoord; \n\
    v_maskTexCoord = vec2(a_maskTexCoord.y + uOffset * a_maskTexCoord.z, a_maskTexCoord.w); \n\
    vec4 pos = vec4(transformedAxisPos, a_position.z, 1.0) * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
  } \n\
";

char const TRANSIT_CIRCLE_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec3 a_position; \n\
  attribute vec4 a_normal; \n\
  attribute vec4 a_color; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_lineHalfWidth; \n\
  uniform float u_maxRadius; \n\
  varying vec3 v_radius; \n\
  varying vec4 v_color; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec2 normal = a_normal.xy; \n\
    vec2 transformedAxisPos = (vec4(a_position.xy, 0.0, 1.0) * u_modelView).xy; \n\
    if (dot(normal, normal) != 0.0) \n\
    { \n\
      vec2 norm = normal * u_lineHalfWidth; \n\
      transformedAxisPos = calcLineTransformedAxisPos(transformedAxisPos, a_position.xy + norm, \n\
                                                      u_modelView, length(norm)); \n\
    } \n\
    transformedAxisPos += a_normal.zw * u_lineHalfWidth; \n\
    vec4 pos = vec4(transformedAxisPos, a_position.z, 1.0) * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
    v_color = a_color; \n\
    v_radius = vec3(a_normal.zw, u_maxRadius) * u_lineHalfWidth; \n\
  } \n\
";

char const GLES3_TRANSIT_CIRCLE_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 a_position; \n\
  in vec4 a_normal; \n\
  in vec4 a_color; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_lineHalfWidth; \n\
  uniform float u_maxRadius; \n\
  out vec3 v_radius; \n\
  out vec4 v_color; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec2 normal = a_normal.xy; \n\
    vec2 transformedAxisPos = (vec4(a_position.xy, 0.0, 1.0) * u_modelView).xy; \n\
    if (dot(normal, normal) != 0.0) \n\
    { \n\
      vec2 norm = normal * u_lineHalfWidth; \n\
      transformedAxisPos = calcLineTransformedAxisPos(transformedAxisPos, a_position.xy + norm, \n\
                                                      u_modelView, length(norm)); \n\
    } \n\
    transformedAxisPos += a_normal.zw * u_lineHalfWidth; \n\
    vec4 pos = vec4(transformedAxisPos, a_position.z, 1.0) * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
    v_color = a_color; \n\
    v_radius = vec3(a_normal.zw, u_maxRadius) * u_lineHalfWidth; \n\
  } \n\
";

char const TEXT_OUTLINED_BILLBOARD_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec2 a_colorTexCoord; \n\
  attribute vec2 a_outlineColorTexCoord; \n\
  attribute vec2 a_maskTexCoord; \n\
  attribute vec4 a_position; \n\
  attribute vec2 a_normal; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_isOutlinePass; \n\
  uniform float u_zScale; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  varying LOW_P vec4 v_color; \n\
  #else \n\
  varying vec2 v_colorTexCoord; \n\
  #endif \n\
  varying vec2 v_maskTexCoord; \n\
  const float kBaseDepthShift = -10.0; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    float isOutline = step(0.5, u_isOutlinePass); \n\
    float depthShift = kBaseDepthShift * isOutline; \n\
   \n\
    vec4 pivot = (vec4(a_position.xyz, 1.0) + vec4(0.0, 0.0, depthShift, 0.0)) * u_modelView; \n\
    vec4 offset = vec4(a_normal, 0.0, 0.0) * u_projection; \n\
    gl_Position = applyBillboardPivotTransform(pivot * u_projection, u_pivotTransform, \n\
                                               a_position.w * u_zScale, offset.xy); \n\
    vec2 colorTexCoord = mix(a_colorTexCoord, a_outlineColorTexCoord, isOutline); \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture2D(u_colorTex, colorTexCoord); \n\
  #else \n\
    v_colorTexCoord = colorTexCoord; \n\
  #endif \n\
    v_maskTexCoord = a_maskTexCoord; \n\
  } \n\
";

char const GLES3_TEXT_OUTLINED_BILLBOARD_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec2 a_colorTexCoord; \n\
  in vec2 a_outlineColorTexCoord; \n\
  in vec2 a_maskTexCoord; \n\
  in vec4 a_position; \n\
  in vec2 a_normal; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_isOutlinePass; \n\
  uniform float u_zScale; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  out LOW_P vec4 v_color; \n\
  #else \n\
  out vec2 v_colorTexCoord; \n\
  #endif \n\
  out vec2 v_maskTexCoord; \n\
  const float kBaseDepthShift = -10.0; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    float isOutline = step(0.5, u_isOutlinePass); \n\
    float depthShift = kBaseDepthShift * isOutline; \n\
   \n\
    vec4 pivot = (vec4(a_position.xyz, 1.0) + vec4(0.0, 0.0, depthShift, 0.0)) * u_modelView; \n\
    vec4 offset = vec4(a_normal, 0.0, 0.0) * u_projection; \n\
    gl_Position = applyBillboardPivotTransform(pivot * u_projection, u_pivotTransform, \n\
                                               a_position.w * u_zScale, offset.xy); \n\
    vec2 colorTexCoord = mix(a_colorTexCoord, a_outlineColorTexCoord, isOutline); \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture(u_colorTex, colorTexCoord); \n\
  #else \n\
    v_colorTexCoord = colorTexCoord; \n\
  #endif \n\
    v_maskTexCoord = a_maskTexCoord; \n\
  } \n\
";

char const SCREEN_QUAD_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec2 a_pos; \n\
  attribute vec2 a_tcoord; \n\
  varying vec2 v_colorTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    v_colorTexCoords = a_tcoord; \n\
    gl_Position = vec4(a_pos, 0.0, 1.0); \n\
  } \n\
";

char const GLES3_SCREEN_QUAD_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec2 a_pos; \n\
  in vec2 a_tcoord; \n\
  out vec2 v_colorTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    v_colorTexCoords = a_tcoord; \n\
    gl_Position = vec4(a_pos, 0.0, 1.0); \n\
  } \n\
";

char const TRAFFIC_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec3 a_position; \n\
  attribute vec4 a_normal; \n\
  attribute vec4 a_colorTexCoord; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform vec4 u_trafficParams; \n\
  varying vec2 v_colorTexCoord; \n\
  varying vec2 v_maskTexCoord; \n\
  varying float v_halfLength; \n\
  const float kArrowVSize = 0.25; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec2 normal = a_normal.xy; \n\
    vec2 transformedAxisPos = (vec4(a_position.xy, 0.0, 1.0) * u_modelView).xy; \n\
    if (dot(normal, normal) != 0.0) \n\
    { \n\
      vec2 norm = normal * u_trafficParams.x; \n\
      if (a_normal.z < 0.0) \n\
        norm = normal * u_trafficParams.y; \n\
      transformedAxisPos = calcLineTransformedAxisPos(transformedAxisPos, a_position.xy + norm, \n\
                                                      u_modelView, length(norm)); \n\
    } \n\
    float uOffset = length(vec4(kShapeCoordScalar, 0, 0, 0) * u_modelView) * a_normal.w; \n\
    v_colorTexCoord = a_colorTexCoord.xy; \n\
    float v = mix(a_colorTexCoord.z, a_colorTexCoord.z + kArrowVSize, 0.5 * a_normal.z + 0.5); \n\
    v_maskTexCoord = vec2(uOffset * u_trafficParams.z, v) * u_trafficParams.w; \n\
    v_maskTexCoord.x *= step(a_colorTexCoord.w, v_maskTexCoord.x); \n\
    v_halfLength = a_normal.z; \n\
    vec4 pos = vec4(transformedAxisPos, a_position.z, 1.0) * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
  } \n\
";

char const GLES3_TRAFFIC_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 a_position; \n\
  in vec4 a_normal; \n\
  in vec4 a_colorTexCoord; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform vec4 u_trafficParams; \n\
  out vec2 v_colorTexCoord; \n\
  out vec2 v_maskTexCoord; \n\
  out float v_halfLength; \n\
  const float kArrowVSize = 0.25; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec2 normal = a_normal.xy; \n\
    vec2 transformedAxisPos = (vec4(a_position.xy, 0.0, 1.0) * u_modelView).xy; \n\
    if (dot(normal, normal) != 0.0) \n\
    { \n\
      vec2 norm = normal * u_trafficParams.x; \n\
      if (a_normal.z < 0.0) \n\
        norm = normal * u_trafficParams.y; \n\
      transformedAxisPos = calcLineTransformedAxisPos(transformedAxisPos, a_position.xy + norm, \n\
                                                      u_modelView, length(norm)); \n\
    } \n\
    float uOffset = length(vec4(kShapeCoordScalar, 0, 0, 0) * u_modelView) * a_normal.w; \n\
    v_colorTexCoord = a_colorTexCoord.xy; \n\
    float v = mix(a_colorTexCoord.z, a_colorTexCoord.z + kArrowVSize, 0.5 * a_normal.z + 0.5); \n\
    v_maskTexCoord = vec2(uOffset * u_trafficParams.z, v) * u_trafficParams.w; \n\
    v_maskTexCoord.x *= step(a_colorTexCoord.w, v_maskTexCoord.x); \n\
    v_halfLength = a_normal.z; \n\
    vec4 pos = vec4(transformedAxisPos, a_position.z, 1.0) * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
  } \n\
";

char const TEXTURING_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec4 a_position; \n\
  attribute vec2 a_normal; \n\
  attribute vec2 a_colorTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  varying vec2 v_colorTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pos = vec4(a_position.xyz, 1) * u_modelView; \n\
    vec4 shiftedPos = vec4(a_normal, 0, 0) + pos; \n\
    gl_Position = applyPivotTransform(shiftedPos * u_projection, u_pivotTransform, 0.0); \n\
    v_colorTexCoords = a_colorTexCoords; \n\
  } \n\
";

char const GLES3_TEXTURING_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec4 a_position; \n\
  in vec2 a_normal; \n\
  in vec2 a_colorTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  out vec2 v_colorTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pos = vec4(a_position.xyz, 1) * u_modelView; \n\
    vec4 shiftedPos = vec4(a_normal, 0, 0) + pos; \n\
    gl_Position = applyPivotTransform(shiftedPos * u_projection, u_pivotTransform, 0.0); \n\
    v_colorTexCoords = a_colorTexCoords; \n\
  } \n\
";

char const ARROW3D_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec3 a_pos; \n\
  attribute vec3 a_normal; \n\
  uniform mat4 u_transform; \n\
  uniform mat4 u_normalTransform; \n\
  varying vec3 v_normal; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 position = u_transform * vec4(a_pos, 1.0); \n\
    v_normal = normalize((u_normalTransform * vec4(a_normal, 0.0)).xyz); \n\
    gl_Position = position; \n\
  #ifdef VULKAN \n\
    gl_Position.y = -gl_Position.y; \n\
    gl_Position.z = (gl_Position.z + gl_Position.w) * 0.5; \n\
  #endif \n\
  } \n\
";

char const GLES3_ARROW3D_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 a_pos; \n\
  in vec3 a_normal; \n\
  uniform mat4 u_transform; \n\
  uniform mat4 u_normalTransform; \n\
  out vec3 v_normal; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 position = u_transform * vec4(a_pos, 1.0); \n\
    v_normal = normalize((u_normalTransform * vec4(a_normal, 0.0)).xyz); \n\
    gl_Position = position; \n\
  #ifdef VULKAN \n\
    gl_Position.y = -gl_Position.y; \n\
    gl_Position.z = (gl_Position.z + gl_Position.w) * 0.5; \n\
  #endif \n\
  } \n\
";

char const SMAA_BLENDING_WEIGHT_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec2 a_pos; \n\
  attribute vec2 a_tcoord; \n\
  uniform vec4 u_framebufferMetrics; \n\
  varying vec4 v_coords; \n\
  varying vec4 v_offset0; \n\
  varying vec4 v_offset1; \n\
  varying vec4 v_offset2; \n\
  #define SMAA_MAX_SEARCH_STEPS 8.0 \n\
  const vec4 kMaxSearchSteps = vec4(-2.0 * SMAA_MAX_SEARCH_STEPS, 2.0 * SMAA_MAX_SEARCH_STEPS, \n\
                                    -2.0 * SMAA_MAX_SEARCH_STEPS, 2.0  * SMAA_MAX_SEARCH_STEPS); \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    v_coords = vec4(a_tcoord, a_tcoord * u_framebufferMetrics.zw); \n\
    v_offset0 = u_framebufferMetrics.xyxy * vec4(-0.25, -0.125, 1.25, -0.125) + a_tcoord.xyxy; \n\
    v_offset1 = u_framebufferMetrics.xyxy * vec4(-0.125, -0.25, -0.125, 1.25) + a_tcoord.xyxy; \n\
    v_offset2 = u_framebufferMetrics.xxyy * kMaxSearchSteps + vec4(v_offset0.xz, v_offset1.yw); \n\
    gl_Position = vec4(a_pos, 0.0, 1.0); \n\
  } \n\
";

char const GLES3_SMAA_BLENDING_WEIGHT_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec2 a_pos; \n\
  in vec2 a_tcoord; \n\
  uniform vec4 u_framebufferMetrics; \n\
  out vec4 v_coords; \n\
  out vec4 v_offset0; \n\
  out vec4 v_offset1; \n\
  out vec4 v_offset2; \n\
  #define SMAA_MAX_SEARCH_STEPS 8.0 \n\
  const vec4 kMaxSearchSteps = vec4(-2.0 * SMAA_MAX_SEARCH_STEPS, 2.0 * SMAA_MAX_SEARCH_STEPS, \n\
                                    -2.0 * SMAA_MAX_SEARCH_STEPS, 2.0  * SMAA_MAX_SEARCH_STEPS); \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    v_coords = vec4(a_tcoord, a_tcoord * u_framebufferMetrics.zw); \n\
    v_offset0 = u_framebufferMetrics.xyxy * vec4(-0.25, -0.125, 1.25, -0.125) + a_tcoord.xyxy; \n\
    v_offset1 = u_framebufferMetrics.xyxy * vec4(-0.125, -0.25, -0.125, 1.25) + a_tcoord.xyxy; \n\
    v_offset2 = u_framebufferMetrics.xxyy * kMaxSearchSteps + vec4(v_offset0.xz, v_offset1.yw); \n\
    gl_Position = vec4(a_pos, 0.0, 1.0); \n\
  } \n\
";

char const ARROW3D_TEXTURED_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec3 a_pos; \n\
  attribute vec3 a_normal; \n\
  attribute vec2 a_texCoords; \n\
  uniform mat4 u_transform; \n\
  uniform mat4 u_normalTransform; \n\
  uniform vec2 u_texCoordFlipping; \n\
  varying vec3 v_normal; \n\
  varying vec2 v_texCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 position = u_transform * vec4(a_pos, 1.0); \n\
    v_normal = normalize((u_normalTransform * vec4(a_normal, 0.0)).xyz); \n\
    v_texCoords = mix(a_texCoords, 1.0 - a_texCoords, u_texCoordFlipping); \n\
    gl_Position = position; \n\
  #ifdef VULKAN \n\
    gl_Position.y = -gl_Position.y; \n\
    gl_Position.z = (gl_Position.z + gl_Position.w) * 0.5; \n\
  #endif \n\
  } \n\
";

char const GLES3_ARROW3D_TEXTURED_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 a_pos; \n\
  in vec3 a_normal; \n\
  in vec2 a_texCoords; \n\
  uniform mat4 u_transform; \n\
  uniform mat4 u_normalTransform; \n\
  uniform vec2 u_texCoordFlipping; \n\
  out vec3 v_normal; \n\
  out vec2 v_texCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 position = u_transform * vec4(a_pos, 1.0); \n\
    v_normal = normalize((u_normalTransform * vec4(a_normal, 0.0)).xyz); \n\
    v_texCoords = mix(a_texCoords, 1.0 - a_texCoords, u_texCoordFlipping); \n\
    gl_Position = position; \n\
  #ifdef VULKAN \n\
    gl_Position.y = -gl_Position.y; \n\
    gl_Position.z = (gl_Position.z + gl_Position.w) * 0.5; \n\
  #endif \n\
  } \n\
";

char const ARROW3D_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  varying vec3 v_normal; \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
  uniform sampler2D u_colorTex; \n\
  #endif \n\
  const vec3 lightDir = vec3(0.316, 0.0, 0.948); \n\
  uniform vec4 u_color; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
    float phongDiffuse = max(0.0, -dot(lightDir, v_normal)); \n\
    vec4 resColor = vec4((phongDiffuse * 0.5 + 0.5) * u_color.rgb, u_color.a); \n\
    gl_FragColor = samsungGoogleNexusWorkaround(resColor); \n\
  } \n\
";

char const GLES3_ARROW3D_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 v_normal; \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
  uniform sampler2D u_colorTex; \n\
  #endif \n\
  const vec3 lightDir = vec3(0.316, 0.0, 0.948); \n\
  uniform vec4 u_color; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
    float phongDiffuse = max(0.0, -dot(lightDir, v_normal)); \n\
    vec4 resColor = vec4((phongDiffuse * 0.5 + 0.5) * u_color.rgb, u_color.a); \n\
    v_FragColor = samsungGoogleNexusWorkaround(resColor); \n\
  } \n\
";

char const SMAA_EDGES_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform sampler2D u_colorTex; \n\
  varying vec2 v_colorTexCoords; \n\
  varying vec4 v_offset0; \n\
  varying vec4 v_offset1; \n\
  varying vec4 v_offset2; \n\
  #define SMAA_THRESHOLD 0.05 \n\
  const vec2 kThreshold = vec2(SMAA_THRESHOLD, SMAA_THRESHOLD); \n\
  #define SMAA_LOCAL_CONTRAST_ADAPTATION_FACTOR 2.0 \n\
  const vec3 kWeights = vec3(0.2126, 0.7152, 0.0722); \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
    float L = dot(texture2D(u_colorTex, v_colorTexCoords).rgb, kWeights); \n\
    float Lleft = dot(texture2D(u_colorTex, v_offset0.xy).rgb, kWeights); \n\
    float Ltop = dot(texture2D(u_colorTex, v_offset0.zw).rgb, kWeights); \n\
    vec4 delta; \n\
    delta.xy = abs(L - vec2(Lleft, Ltop)); \n\
    vec2 edges = step(kThreshold, delta.xy); \n\
    if (dot(edges, vec2(1.0, 1.0)) == 0.0) \n\
        discard; \n\
    float Lright = dot(texture2D(u_colorTex, v_offset1.xy).rgb, kWeights); \n\
    float Lbottom  = dot(texture2D(u_colorTex, v_offset1.zw).rgb, kWeights); \n\
    delta.zw = abs(L - vec2(Lright, Lbottom)); \n\
    vec2 maxDelta = max(delta.xy, delta.zw); \n\
    float Lleftleft = dot(texture2D(u_colorTex, v_offset2.xy).rgb, kWeights); \n\
    float Ltoptop = dot(texture2D(u_colorTex, v_offset2.zw).rgb, kWeights); \n\
    delta.zw = abs(vec2(Lleft, Ltop) - vec2(Lleftleft, Ltoptop)); \n\
    maxDelta = max(maxDelta.xy, delta.zw); \n\
    float finalDelta = max(maxDelta.x, maxDelta.y); \n\
    edges *= step(finalDelta, SMAA_LOCAL_CONTRAST_ADAPTATION_FACTOR * delta.xy); \n\
    gl_FragColor = vec4(edges, 0.0, 1.0); \n\
  } \n\
";

char const GLES3_SMAA_EDGES_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform sampler2D u_colorTex; \n\
  in vec2 v_colorTexCoords; \n\
  in vec4 v_offset0; \n\
  in vec4 v_offset1; \n\
  in vec4 v_offset2; \n\
  #define SMAA_THRESHOLD 0.05 \n\
  const vec2 kThreshold = vec2(SMAA_THRESHOLD, SMAA_THRESHOLD); \n\
  #define SMAA_LOCAL_CONTRAST_ADAPTATION_FACTOR 2.0 \n\
  const vec3 kWeights = vec3(0.2126, 0.7152, 0.0722); \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
    float L = dot(texture(u_colorTex, v_colorTexCoords).rgb, kWeights); \n\
    float Lleft = dot(texture(u_colorTex, v_offset0.xy).rgb, kWeights); \n\
    float Ltop = dot(texture(u_colorTex, v_offset0.zw).rgb, kWeights); \n\
    vec4 delta; \n\
    delta.xy = abs(L - vec2(Lleft, Ltop)); \n\
    vec2 edges = step(kThreshold, delta.xy); \n\
    if (dot(edges, vec2(1.0, 1.0)) == 0.0) \n\
        discard; \n\
    float Lright = dot(texture(u_colorTex, v_offset1.xy).rgb, kWeights); \n\
    float Lbottom  = dot(texture(u_colorTex, v_offset1.zw).rgb, kWeights); \n\
    delta.zw = abs(L - vec2(Lright, Lbottom)); \n\
    vec2 maxDelta = max(delta.xy, delta.zw); \n\
    float Lleftleft = dot(texture(u_colorTex, v_offset2.xy).rgb, kWeights); \n\
    float Ltoptop = dot(texture(u_colorTex, v_offset2.zw).rgb, kWeights); \n\
    delta.zw = abs(vec2(Lleft, Ltop) - vec2(Lleftleft, Ltoptop)); \n\
    maxDelta = max(maxDelta.xy, delta.zw); \n\
    float finalDelta = max(maxDelta.x, maxDelta.y); \n\
    edges *= step(finalDelta, SMAA_LOCAL_CONTRAST_ADAPTATION_FACTOR * delta.xy); \n\
    v_FragColor = vec4(edges, 0.0, 1.0); \n\
  } \n\
";

char const ROUTE_DASH_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  varying vec3 v_length; \n\
  varying vec4 v_color; \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
  uniform sampler2D u_colorTex; \n\
  #endif \n\
  uniform vec4 u_color; \n\
  uniform vec2 u_pattern; \n\
  uniform vec4 u_maskColor; \n\
  uniform vec2 u_fakeBorders; \n\
  uniform vec4 u_fakeColor; \n\
  const float kAntialiasingThreshold = 0.92; \n\
  float alphaFromPattern(float curLen, float dashLen, float gapLen) \n\
  { \n\
    float len = dashLen + gapLen; \n\
    float offset = fract(curLen / len) * len; \n\
    return step(offset, dashLen); \n\
  } \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
    if (v_length.x < v_length.z) \n\
      discard; \n\
    vec2 coefs = step(v_length.xx, u_fakeBorders); \n\
    coefs.y = 1.0 - coefs.y; \n\
    vec4 mainColor = mix(u_color, u_fakeColor, coefs.x); \n\
    mainColor = mix(mainColor, u_fakeColor, coefs.y); \n\
    vec4 color = mainColor + v_color; \n\
    color.a *= (1.0 - smoothstep(kAntialiasingThreshold, 1.0, abs(v_length.y))) * \n\
                alphaFromPattern(v_length.x, u_pattern.x, u_pattern.y); \n\
    color = vec4(mix(color.rgb, u_maskColor.rgb, u_maskColor.a), color.a); \n\
    gl_FragColor = samsungGoogleNexusWorkaround(color); \n\
  } \n\
";

char const GLES3_ROUTE_DASH_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 v_length; \n\
  in vec4 v_color; \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
  uniform sampler2D u_colorTex; \n\
  #endif \n\
  uniform vec4 u_color; \n\
  uniform vec2 u_pattern; \n\
  uniform vec4 u_maskColor; \n\
  uniform vec2 u_fakeBorders; \n\
  uniform vec4 u_fakeColor; \n\
  const float kAntialiasingThreshold = 0.92; \n\
  float alphaFromPattern(float curLen, float dashLen, float gapLen) \n\
  { \n\
    float len = dashLen + gapLen; \n\
    float offset = fract(curLen / len) * len; \n\
    return step(offset, dashLen); \n\
  } \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
    if (v_length.x < v_length.z) \n\
      discard; \n\
    vec2 coefs = step(v_length.xx, u_fakeBorders); \n\
    coefs.y = 1.0 - coefs.y; \n\
    vec4 mainColor = mix(u_color, u_fakeColor, coefs.x); \n\
    mainColor = mix(mainColor, u_fakeColor, coefs.y); \n\
    vec4 color = mainColor + v_color; \n\
    color.a *= (1.0 - smoothstep(kAntialiasingThreshold, 1.0, abs(v_length.y))) * \n\
                alphaFromPattern(v_length.x, u_pattern.x, u_pattern.y); \n\
    color = vec4(mix(color.rgb, u_maskColor.rgb, u_maskColor.a), color.a); \n\
    v_FragColor = samsungGoogleNexusWorkaround(color); \n\
  } \n\
";

char const ROUTE_ARROW_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform sampler2D u_colorTex; \n\
  uniform float u_opacity; \n\
  uniform vec4 u_maskColor; \n\
  varying vec2 v_colorTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
    vec4 finalColor = texture2D(u_colorTex, v_colorTexCoords); \n\
    finalColor.a *= u_opacity; \n\
    if (finalColor.a < 0.01) \n\
      discard; \n\
    finalColor = vec4(mix(finalColor.rgb, u_maskColor.rgb, u_maskColor.a), finalColor.a); \n\
    gl_FragColor = finalColor; \n\
  } \n\
";

char const GLES3_ROUTE_ARROW_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform sampler2D u_colorTex; \n\
  uniform float u_opacity; \n\
  uniform vec4 u_maskColor; \n\
  in vec2 v_colorTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
    vec4 finalColor = texture(u_colorTex, v_colorTexCoords); \n\
    finalColor.a *= u_opacity; \n\
    if (finalColor.a < 0.01) \n\
      discard; \n\
    finalColor = vec4(mix(finalColor.rgb, u_maskColor.rgb, u_maskColor.a), finalColor.a); \n\
    v_FragColor = finalColor; \n\
  } \n\
";

char const TEXTURING3D_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform sampler2D u_colorTex; \n\
  uniform float u_opacity; \n\
  varying vec2 v_colorTexCoords; \n\
  varying float v_intensity; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
    vec4 finalColor = vec4(texture2D(u_colorTex, v_colorTexCoords).rgb, u_opacity); \n\
    gl_FragColor = vec4((v_intensity * 0.2 + 0.8) * finalColor.rgb, finalColor.a); \n\
  } \n\
";

char const GLES3_TEXTURING3D_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform sampler2D u_colorTex; \n\
  uniform float u_opacity; \n\
  in vec2 v_colorTexCoords; \n\
  in float v_intensity; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
    vec4 finalColor = vec4(texture(u_colorTex, v_colorTexCoords).rgb, u_opacity); \n\
    v_FragColor = vec4((v_intensity * 0.2 + 0.8) * finalColor.rgb, finalColor.a); \n\
  } \n\
";

char const AREA_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec3 a_position; \n\
  attribute vec2 a_colorTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  varying LOW_P vec4 v_color; \n\
  #else \n\
  varying vec2 v_colorTexCoords; \n\
  #endif \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pos = vec4(a_position, 1) * u_modelView * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture2D(u_colorTex, a_colorTexCoords); \n\
  #else \n\
    v_colorTexCoords = a_colorTexCoords; \n\
  #endif \n\
  } \n\
";

char const GLES3_AREA_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 a_position; \n\
  in vec2 a_colorTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  out LOW_P vec4 v_color; \n\
  #else \n\
  out vec2 v_colorTexCoords; \n\
  #endif \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pos = vec4(a_position, 1) * u_modelView * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture(u_colorTex, a_colorTexCoords); \n\
  #else \n\
    v_colorTexCoords = a_colorTexCoords; \n\
  #endif \n\
  } \n\
";

char const MASKED_TEXTURING_BILLBOARD_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec4 a_position; \n\
  attribute vec2 a_normal; \n\
  attribute vec2 a_colorTexCoords; \n\
  attribute vec2 a_maskTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_zScale; \n\
  varying vec2 v_colorTexCoords; \n\
  varying vec2 v_maskTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pivot = vec4(a_position.xyz, 1.0) * u_modelView; \n\
    vec4 offset = vec4(a_normal, 0.0, 0.0) * u_projection; \n\
    gl_Position = applyBillboardPivotTransform(pivot * u_projection, u_pivotTransform, \n\
                                               a_position.w * u_zScale, offset.xy); \n\
    v_colorTexCoords = a_colorTexCoords; \n\
    v_maskTexCoords = a_maskTexCoords; \n\
  } \n\
";

char const GLES3_MASKED_TEXTURING_BILLBOARD_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec4 a_position; \n\
  in vec2 a_normal; \n\
  in vec2 a_colorTexCoords; \n\
  in vec2 a_maskTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_zScale; \n\
  out vec2 v_colorTexCoords; \n\
  out vec2 v_maskTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pivot = vec4(a_position.xyz, 1.0) * u_modelView; \n\
    vec4 offset = vec4(a_normal, 0.0, 0.0) * u_projection; \n\
    gl_Position = applyBillboardPivotTransform(pivot * u_projection, u_pivotTransform, \n\
                                               a_position.w * u_zScale, offset.xy); \n\
    v_colorTexCoords = a_colorTexCoords; \n\
    v_maskTexCoords = a_maskTexCoords; \n\
  } \n\
";

char const SMAA_FINAL_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform sampler2D u_colorTex; \n\
  uniform sampler2D u_blendingWeightTex; \n\
  uniform vec4 u_framebufferMetrics; \n\
  varying vec2 v_colorTexCoords; \n\
  varying vec4 v_offset; \n\
  #ifdef GLES3 \n\
    #define SMAASampleLevelZero(tex, coord) textureLod(tex, coord, 0.0) \n\
  #else \n\
    #define SMAASampleLevelZero(tex, coord) texture2D(tex, coord) \n\
  #endif \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
    vec4 a; \n\
    a.x = texture2D(u_blendingWeightTex, v_offset.xy).a; // Right \n\
    a.y = texture2D(u_blendingWeightTex, v_offset.zw).g; // Top \n\
    a.wz = texture2D(u_blendingWeightTex, v_colorTexCoords).xz; // Bottom / Left \n\
    if (dot(a, vec4(1.0, 1.0, 1.0, 1.0)) < 1e-5) \n\
    { \n\
      gl_FragColor = texture2D(u_colorTex, v_colorTexCoords); \n\
    } \n\
    else \n\
    { \n\
      vec4 blendingOffset = vec4(0.0, a.y, 0.0, a.w); \n\
      vec2 blendingWeight = a.yw; \n\
      if (max(a.x, a.z) > max(a.y, a.w)) \n\
      { \n\
        blendingOffset = vec4(a.x, 0.0, a.z, 0.0); \n\
        blendingWeight = a.xz; \n\
      } \n\
      blendingWeight /= dot(blendingWeight, vec2(1.0, 1.0)); \n\
      vec4 bc = blendingOffset * vec4(u_framebufferMetrics.xy, -u_framebufferMetrics.xy); \n\
      bc += v_colorTexCoords.xyxy; \n\
      vec4 color = blendingWeight.x * SMAASampleLevelZero(u_colorTex, bc.xy); \n\
      color += blendingWeight.y * SMAASampleLevelZero(u_colorTex, bc.zw); \n\
      gl_FragColor = color; \n\
    } \n\
  } \n\
";

char const GLES3_SMAA_FINAL_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform sampler2D u_colorTex; \n\
  uniform sampler2D u_blendingWeightTex; \n\
  uniform vec4 u_framebufferMetrics; \n\
  in vec2 v_colorTexCoords; \n\
  in vec4 v_offset; \n\
  #ifdef GLES3 \n\
    #define SMAASampleLevelZero(tex, coord) textureLod(tex, coord, 0.0) \n\
  #else \n\
    #define SMAASampleLevelZero(tex, coord) texture(tex, coord) \n\
  #endif \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
    vec4 a; \n\
    a.x = texture(u_blendingWeightTex, v_offset.xy).a; // Right \n\
    a.y = texture(u_blendingWeightTex, v_offset.zw).g; // Top \n\
    a.wz = texture(u_blendingWeightTex, v_colorTexCoords).xz; // Bottom / Left \n\
    if (dot(a, vec4(1.0, 1.0, 1.0, 1.0)) < 1e-5) \n\
    { \n\
      v_FragColor = texture(u_colorTex, v_colorTexCoords); \n\
    } \n\
    else \n\
    { \n\
      vec4 blendingOffset = vec4(0.0, a.y, 0.0, a.w); \n\
      vec2 blendingWeight = a.yw; \n\
      if (max(a.x, a.z) > max(a.y, a.w)) \n\
      { \n\
        blendingOffset = vec4(a.x, 0.0, a.z, 0.0); \n\
        blendingWeight = a.xz; \n\
      } \n\
      blendingWeight /= dot(blendingWeight, vec2(1.0, 1.0)); \n\
      vec4 bc = blendingOffset * vec4(u_framebufferMetrics.xy, -u_framebufferMetrics.xy); \n\
      bc += v_colorTexCoords.xyxy; \n\
      vec4 color = blendingWeight.x * SMAASampleLevelZero(u_colorTex, bc.xy); \n\
      color += blendingWeight.y * SMAASampleLevelZero(u_colorTex, bc.zw); \n\
      v_FragColor = color; \n\
    } \n\
  } \n\
";

char const ARROW3D_TEXTURED_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  varying vec3 v_normal; \n\
  varying vec2 v_texCoords; \n\
  uniform sampler2D u_colorTex; \n\
  const vec3 lightDir = vec3(0.316, 0.0, 0.948); \n\
  uniform vec4 u_color; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
    float phongDiffuse = max(0.0, -dot(lightDir, v_normal)); \n\
    vec4 color = texture2D(u_colorTex, v_texCoords) * u_color; \n\
    gl_FragColor = vec4((phongDiffuse * 0.5 + 0.5) * color.rgb, color.a); \n\
  } \n\
";

char const GLES3_ARROW3D_TEXTURED_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 v_normal; \n\
  in vec2 v_texCoords; \n\
  uniform sampler2D u_colorTex; \n\
  const vec3 lightDir = vec3(0.316, 0.0, 0.948); \n\
  uniform vec4 u_color; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
    float phongDiffuse = max(0.0, -dot(lightDir, v_normal)); \n\
    vec4 color = texture(u_colorTex, v_texCoords) * u_color; \n\
    v_FragColor = vec4((phongDiffuse * 0.5 + 0.5) * color.rgb, color.a); \n\
  } \n\
";

char const SELECTION_LINE_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec3 a_position; \n\
  attribute vec2 a_normal; \n\
  attribute vec2 a_colorTexCoords; \n\
  attribute vec3 a_length; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform vec2 u_lineParams; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  varying LOW_P vec4 v_color; \n\
  #else \n\
  varying vec2 v_colorTexCoord; \n\
  #endif \n\
  varying float v_lengthY; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec2 transformedAxisPos = (vec4(a_position.xy, 0.0, 1.0) * u_modelView).xy; \n\
    vec2 len = vec2(a_length.x, a_length.z); \n\
    if (dot(a_normal, a_normal) != 0.0) \n\
    { \n\
      vec2 norm = a_normal * u_lineParams.x; \n\
      transformedAxisPos = calcLineTransformedAxisPos(transformedAxisPos, a_position.xy + norm, \n\
                                                      u_modelView, length(norm)); \n\
      if (u_lineParams.y != 0.0) \n\
        len = vec2(a_length.x + a_length.y * u_lineParams.y, a_length.z); \n\
    } \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture2D(u_colorTex, a_colorTexCoords); \n\
  #else \n\
    v_colorTexCoord = a_colorTexCoords; \n\
  #endif \n\
    v_lengthY = len.y; \n\
    vec4 pos = vec4(transformedAxisPos, a_position.z, 1.0) * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
  } \n\
";

char const GLES3_SELECTION_LINE_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 a_position; \n\
  in vec2 a_normal; \n\
  in vec2 a_colorTexCoords; \n\
  in vec3 a_length; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform vec2 u_lineParams; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  out LOW_P vec4 v_color; \n\
  #else \n\
  out vec2 v_colorTexCoord; \n\
  #endif \n\
  out float v_lengthY; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec2 transformedAxisPos = (vec4(a_position.xy, 0.0, 1.0) * u_modelView).xy; \n\
    vec2 len = vec2(a_length.x, a_length.z); \n\
    if (dot(a_normal, a_normal) != 0.0) \n\
    { \n\
      vec2 norm = a_normal * u_lineParams.x; \n\
      transformedAxisPos = calcLineTransformedAxisPos(transformedAxisPos, a_position.xy + norm, \n\
                                                      u_modelView, length(norm)); \n\
      if (u_lineParams.y != 0.0) \n\
        len = vec2(a_length.x + a_length.y * u_lineParams.y, a_length.z); \n\
    } \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture(u_colorTex, a_colorTexCoords); \n\
  #else \n\
    v_colorTexCoord = a_colorTexCoords; \n\
  #endif \n\
    v_lengthY = len.y; \n\
    vec4 pos = vec4(transformedAxisPos, a_position.z, 1.0) * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
  } \n\
";

char const ROUTE_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  varying vec3 v_length; \n\
  varying vec4 v_color; \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
  uniform sampler2D u_colorTex; \n\
  #endif \n\
  uniform vec4 u_color; \n\
  uniform vec4 u_outlineColor; \n\
  uniform vec4 u_routeParams; \n\
  uniform vec4 u_maskColor; \n\
  uniform vec2 u_fakeBorders; \n\
  uniform vec4 u_fakeColor; \n\
  uniform vec4 u_fakeOutlineColor; \n\
  const float kAntialiasingThreshold = 0.92; \n\
  const float kOutlineThreshold1 = 0.81; \n\
  const float kOutlineThreshold2 = 0.71; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
    if (v_length.x < v_length.z) \n\
      discard; \n\
    vec2 coefs = step(v_length.xx, u_fakeBorders); \n\
    coefs.y = 1.0 - coefs.y; \n\
    vec4 mainColor = mix(u_color, u_fakeColor, coefs.x); \n\
    mainColor = mix(mainColor, u_fakeColor, coefs.y); \n\
    vec4 mainOutlineColor = mix(u_outlineColor, u_fakeOutlineColor, coefs.x); \n\
    mainOutlineColor = mix(mainOutlineColor, u_fakeOutlineColor, coefs.y); \n\
    vec4 color = mix(mix(mainColor, vec4(v_color.rgb, 1.0), v_color.a), mainColor, step(u_routeParams.w, 0.0)); \n\
    color = mix(color, mainOutlineColor, step(kOutlineThreshold1, abs(v_length.y))); \n\
    color = mix(color, mainOutlineColor, smoothstep(kOutlineThreshold2, kOutlineThreshold1, abs(v_length.y))); \n\
    color.a *= (1.0 - smoothstep(kAntialiasingThreshold, 1.0, abs(v_length.y))); \n\
    color = vec4(mix(color.rgb, u_maskColor.rgb, u_maskColor.a), color.a); \n\
    gl_FragColor = samsungGoogleNexusWorkaround(color); \n\
  } \n\
";

char const GLES3_ROUTE_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 v_length; \n\
  in vec4 v_color; \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
  uniform sampler2D u_colorTex; \n\
  #endif \n\
  uniform vec4 u_color; \n\
  uniform vec4 u_outlineColor; \n\
  uniform vec4 u_routeParams; \n\
  uniform vec4 u_maskColor; \n\
  uniform vec2 u_fakeBorders; \n\
  uniform vec4 u_fakeColor; \n\
  uniform vec4 u_fakeOutlineColor; \n\
  const float kAntialiasingThreshold = 0.92; \n\
  const float kOutlineThreshold1 = 0.81; \n\
  const float kOutlineThreshold2 = 0.71; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
    if (v_length.x < v_length.z) \n\
      discard; \n\
    vec2 coefs = step(v_length.xx, u_fakeBorders); \n\
    coefs.y = 1.0 - coefs.y; \n\
    vec4 mainColor = mix(u_color, u_fakeColor, coefs.x); \n\
    mainColor = mix(mainColor, u_fakeColor, coefs.y); \n\
    vec4 mainOutlineColor = mix(u_outlineColor, u_fakeOutlineColor, coefs.x); \n\
    mainOutlineColor = mix(mainOutlineColor, u_fakeOutlineColor, coefs.y); \n\
    vec4 color = mix(mix(mainColor, vec4(v_color.rgb, 1.0), v_color.a), mainColor, step(u_routeParams.w, 0.0)); \n\
    color = mix(color, mainOutlineColor, step(kOutlineThreshold1, abs(v_length.y))); \n\
    color = mix(color, mainOutlineColor, smoothstep(kOutlineThreshold2, kOutlineThreshold1, abs(v_length.y))); \n\
    color.a *= (1.0 - smoothstep(kAntialiasingThreshold, 1.0, abs(v_length.y))); \n\
    color = vec4(mix(color.rgb, u_maskColor.rgb, u_maskColor.a), color.a); \n\
    v_FragColor = samsungGoogleNexusWorkaround(color); \n\
  } \n\
";

char const TEXT_BILLBOARD_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec2 a_colorTexCoord; \n\
  attribute vec2 a_maskTexCoord; \n\
  attribute vec4 a_position; \n\
  attribute vec2 a_normal; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_isOutlinePass; \n\
  uniform float u_zScale; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  varying LOW_P vec4 v_color; \n\
  #else \n\
  varying vec2 v_colorTexCoord; \n\
  #endif \n\
  varying vec2 v_maskTexCoord; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pivot = vec4(a_position.xyz, 1.0) * u_modelView; \n\
    vec4 offset = vec4(a_normal, 0.0, 0.0) * u_projection; \n\
    gl_Position = applyBillboardPivotTransform(pivot * u_projection, u_pivotTransform, \n\
                                               a_position.w * u_zScale, offset.xy); \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture2D(u_colorTex, a_colorTexCoord); \n\
  #else \n\
    v_colorTexCoord = a_colorTexCoord; \n\
  #endif \n\
    v_maskTexCoord = a_maskTexCoord; \n\
  } \n\
";

char const GLES3_TEXT_BILLBOARD_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec2 a_colorTexCoord; \n\
  in vec2 a_maskTexCoord; \n\
  in vec4 a_position; \n\
  in vec2 a_normal; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_isOutlinePass; \n\
  uniform float u_zScale; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  out LOW_P vec4 v_color; \n\
  #else \n\
  out vec2 v_colorTexCoord; \n\
  #endif \n\
  out vec2 v_maskTexCoord; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pivot = vec4(a_position.xyz, 1.0) * u_modelView; \n\
    vec4 offset = vec4(a_normal, 0.0, 0.0) * u_projection; \n\
    gl_Position = applyBillboardPivotTransform(pivot * u_projection, u_pivotTransform, \n\
                                               a_position.w * u_zScale, offset.xy); \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture(u_colorTex, a_colorTexCoord); \n\
  #else \n\
    v_colorTexCoord = a_colorTexCoord; \n\
  #endif \n\
    v_maskTexCoord = a_maskTexCoord; \n\
  } \n\
";

char const ROUTE_MARKER_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec4 a_position; \n\
  attribute vec3 a_normal; \n\
  attribute vec4 a_color; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform vec2 u_angleCosSin; \n\
  uniform vec4 u_routeParams; \n\
  varying vec4 v_radius; \n\
  varying vec4 v_color; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    float r = u_routeParams.x * a_normal.z; \n\
    vec2 normal = vec2(a_normal.x * u_angleCosSin.x - a_normal.y * u_angleCosSin.y, \n\
                       a_normal.x * u_angleCosSin.y + a_normal.y * u_angleCosSin.x); \n\
    vec4 radius = vec4(normal.xy * r, r, a_position.w); \n\
    vec4 pos = vec4(a_position.xy, 0, 1) * u_modelView; \n\
    vec2 shiftedPos = radius.xy + pos.xy; \n\
    pos = vec4(shiftedPos, a_position.z, 1.0) * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
    v_radius = radius; \n\
    v_color = a_color; \n\
  } \n\
";

char const GLES3_ROUTE_MARKER_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec4 a_position; \n\
  in vec3 a_normal; \n\
  in vec4 a_color; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform vec2 u_angleCosSin; \n\
  uniform vec4 u_routeParams; \n\
  out vec4 v_radius; \n\
  out vec4 v_color; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    float r = u_routeParams.x * a_normal.z; \n\
    vec2 normal = vec2(a_normal.x * u_angleCosSin.x - a_normal.y * u_angleCosSin.y, \n\
                       a_normal.x * u_angleCosSin.y + a_normal.y * u_angleCosSin.x); \n\
    vec4 radius = vec4(normal.xy * r, r, a_position.w); \n\
    vec4 pos = vec4(a_position.xy, 0, 1) * u_modelView; \n\
    vec2 shiftedPos = radius.xy + pos.xy; \n\
    pos = vec4(shiftedPos, a_position.z, 1.0) * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
    v_radius = radius; \n\
    v_color = a_color; \n\
  } \n\
";

char const TEXT_OUTLINED_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec2 a_colorTexCoord; \n\
  attribute vec2 a_outlineColorTexCoord; \n\
  attribute vec2 a_maskTexCoord; \n\
  attribute vec4 a_position; \n\
  attribute vec2 a_normal; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_isOutlinePass; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  varying LOW_P vec4 v_color; \n\
  #else \n\
  varying vec2 v_colorTexCoord; \n\
  #endif \n\
  varying vec2 v_maskTexCoord; \n\
  const float BaseDepthShift = -10.0; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    float isOutline = step(0.5, u_isOutlinePass); \n\
    float notOutline = 1.0 - isOutline; \n\
    float depthShift = BaseDepthShift * isOutline; \n\
    vec4 pos = (vec4(a_position.xyz, 1) + vec4(0.0, 0.0, depthShift, 0.0)) * u_modelView; \n\
    vec4 shiftedPos = vec4(a_normal, 0.0, 0.0) + pos; \n\
    gl_Position = applyPivotTransform(shiftedPos * u_projection, u_pivotTransform, 0.0); \n\
    vec2 colorTexCoord = a_colorTexCoord * notOutline + a_outlineColorTexCoord * isOutline; \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture2D(u_colorTex, colorTexCoord); \n\
  #else \n\
    v_colorTexCoord = colorTexCoord; \n\
  #endif \n\
    v_maskTexCoord = a_maskTexCoord; \n\
  } \n\
";

char const GLES3_TEXT_OUTLINED_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec2 a_colorTexCoord; \n\
  in vec2 a_outlineColorTexCoord; \n\
  in vec2 a_maskTexCoord; \n\
  in vec4 a_position; \n\
  in vec2 a_normal; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_isOutlinePass; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  out LOW_P vec4 v_color; \n\
  #else \n\
  out vec2 v_colorTexCoord; \n\
  #endif \n\
  out vec2 v_maskTexCoord; \n\
  const float BaseDepthShift = -10.0; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    float isOutline = step(0.5, u_isOutlinePass); \n\
    float notOutline = 1.0 - isOutline; \n\
    float depthShift = BaseDepthShift * isOutline; \n\
    vec4 pos = (vec4(a_position.xyz, 1) + vec4(0.0, 0.0, depthShift, 0.0)) * u_modelView; \n\
    vec4 shiftedPos = vec4(a_normal, 0.0, 0.0) + pos; \n\
    gl_Position = applyPivotTransform(shiftedPos * u_projection, u_pivotTransform, 0.0); \n\
    vec2 colorTexCoord = a_colorTexCoord * notOutline + a_outlineColorTexCoord * isOutline; \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture(u_colorTex, colorTexCoord); \n\
  #else \n\
    v_colorTexCoord = colorTexCoord; \n\
  #endif \n\
    v_maskTexCoord = a_maskTexCoord; \n\
  } \n\
";

char const ARROW3D_OUTLINE_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  varying float v_intensity; \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
  uniform sampler2D u_colorTex; \n\
  #endif \n\
  uniform vec4 u_color; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
    vec4 resColor = vec4(u_color.rgb, u_color.a * smoothstep(0.7, 1.0, v_intensity)); \n\
    gl_FragColor = samsungGoogleNexusWorkaround(resColor); \n\
  } \n\
";

char const GLES3_ARROW3D_OUTLINE_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in float v_intensity; \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
  uniform sampler2D u_colorTex; \n\
  #endif \n\
  uniform vec4 u_color; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
    vec4 resColor = vec4(u_color.rgb, u_color.a * smoothstep(0.7, 1.0, v_intensity)); \n\
    v_FragColor = samsungGoogleNexusWorkaround(resColor); \n\
  } \n\
";

char const USER_MARK_BILLBOARD_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec3 a_position; \n\
  attribute vec3 a_normalAndAnimateOrZ; \n\
  attribute vec4 a_texCoords; \n\
  attribute vec4 a_color; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_interpolation; \n\
  varying vec4 v_texCoords; \n\
  varying vec4 v_maskColor; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec2 normal = a_normalAndAnimateOrZ.xy; \n\
    if (a_normalAndAnimateOrZ.z > 0.0) \n\
      normal = u_interpolation * normal; \n\
    vec4 pivot = vec4(a_position, 1.0) * u_modelView; \n\
    vec4 offset = vec4(normal, 0.0, 0.0) * u_projection; \n\
    vec4 projectedPivot = pivot * u_projection; \n\
    gl_Position = applyBillboardPivotTransform(projectedPivot, u_pivotTransform, 0.0, offset.xy); \n\
    float newZ = projectedPivot.y / projectedPivot.w * 0.5 + 0.5; \n\
    gl_Position.z = abs(a_normalAndAnimateOrZ.z) * newZ  + (1.0 - abs(a_normalAndAnimateOrZ.z)) * gl_Position.z; \n\
    v_texCoords = a_texCoords; \n\
    v_maskColor = a_color; \n\
  } \n\
";

char const GLES3_USER_MARK_BILLBOARD_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 a_position; \n\
  in vec3 a_normalAndAnimateOrZ; \n\
  in vec4 a_texCoords; \n\
  in vec4 a_color; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_interpolation; \n\
  out vec4 v_texCoords; \n\
  out vec4 v_maskColor; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec2 normal = a_normalAndAnimateOrZ.xy; \n\
    if (a_normalAndAnimateOrZ.z > 0.0) \n\
      normal = u_interpolation * normal; \n\
    vec4 pivot = vec4(a_position, 1.0) * u_modelView; \n\
    vec4 offset = vec4(normal, 0.0, 0.0) * u_projection; \n\
    vec4 projectedPivot = pivot * u_projection; \n\
    gl_Position = applyBillboardPivotTransform(projectedPivot, u_pivotTransform, 0.0, offset.xy); \n\
    float newZ = projectedPivot.y / projectedPivot.w * 0.5 + 0.5; \n\
    gl_Position.z = abs(a_normalAndAnimateOrZ.z) * newZ  + (1.0 - abs(a_normalAndAnimateOrZ.z)) * gl_Position.z; \n\
    v_texCoords = a_texCoords; \n\
    v_maskColor = a_color; \n\
  } \n\
";

char const TRANSIT_CIRCLE_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  varying vec3 v_radius; \n\
  varying vec4 v_color; \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
  uniform sampler2D u_colorTex; \n\
  #endif \n\
  const float aaPixelsCount = 2.5; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
    vec4 finalColor = v_color; \n\
    float smallRadius = v_radius.z - aaPixelsCount; \n\
    float stepValue = smoothstep(smallRadius * smallRadius, v_radius.z * v_radius.z, \n\
                                dot(v_radius.xy, v_radius.xy)); \n\
    finalColor.a = finalColor.a * (1.0 - stepValue); \n\
    if (finalColor.a < 0.01) \n\
      discard; \n\
    gl_FragColor = samsungGoogleNexusWorkaround(finalColor); \n\
  } \n\
";

char const GLES3_TRANSIT_CIRCLE_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 v_radius; \n\
  in vec4 v_color; \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
  uniform sampler2D u_colorTex; \n\
  #endif \n\
  const float aaPixelsCount = 2.5; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
    vec4 finalColor = v_color; \n\
    float smallRadius = v_radius.z - aaPixelsCount; \n\
    float stepValue = smoothstep(smallRadius * smallRadius, v_radius.z * v_radius.z, \n\
                                dot(v_radius.xy, v_radius.xy)); \n\
    finalColor.a = finalColor.a * (1.0 - stepValue); \n\
    if (finalColor.a < 0.01) \n\
      discard; \n\
    v_FragColor = samsungGoogleNexusWorkaround(finalColor); \n\
  } \n\
";

char const TEXT_OUTLINED_GUI_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec3 a_position; \n\
  attribute vec2 a_colorTexCoord; \n\
  attribute vec2 a_outlineColorTexCoord; \n\
  attribute vec2 a_normal; \n\
  attribute vec2 a_maskTexCoord; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform float u_isOutlinePass; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  varying LOW_P vec4 v_color; \n\
  #else \n\
  varying vec2 v_colorTexCoord; \n\
  #endif \n\
  varying vec2 v_maskTexCoord; \n\
  const float kBaseDepthShift = -10.0; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    float isOutline = step(0.5, u_isOutlinePass); \n\
    float depthShift = kBaseDepthShift * isOutline; \n\
   \n\
    vec4 pos = (vec4(a_position, 1.0) + vec4(0.0, 0.0, depthShift, 0.0)) * u_modelView; \n\
    vec4 shiftedPos = vec4(a_normal, 0.0, 0.0) + pos; \n\
    gl_Position = shiftedPos * u_projection; \n\
  #ifdef VULKAN \n\
    gl_Position.y = -gl_Position.y; \n\
    gl_Position.z = (gl_Position.z  + gl_Position.w) * 0.5; \n\
  #endif \n\
    vec2 colorTexCoord = mix(a_colorTexCoord, a_outlineColorTexCoord, isOutline); \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture2D(u_colorTex, colorTexCoord); \n\
  #else \n\
    v_colorTexCoord = colorTexCoord; \n\
  #endif \n\
    v_maskTexCoord = a_maskTexCoord; \n\
  } \n\
";

char const GLES3_TEXT_OUTLINED_GUI_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 a_position; \n\
  in vec2 a_colorTexCoord; \n\
  in vec2 a_outlineColorTexCoord; \n\
  in vec2 a_normal; \n\
  in vec2 a_maskTexCoord; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform float u_isOutlinePass; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  out LOW_P vec4 v_color; \n\
  #else \n\
  out vec2 v_colorTexCoord; \n\
  #endif \n\
  out vec2 v_maskTexCoord; \n\
  const float kBaseDepthShift = -10.0; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    float isOutline = step(0.5, u_isOutlinePass); \n\
    float depthShift = kBaseDepthShift * isOutline; \n\
   \n\
    vec4 pos = (vec4(a_position, 1.0) + vec4(0.0, 0.0, depthShift, 0.0)) * u_modelView; \n\
    vec4 shiftedPos = vec4(a_normal, 0.0, 0.0) + pos; \n\
    gl_Position = shiftedPos * u_projection; \n\
  #ifdef VULKAN \n\
    gl_Position.y = -gl_Position.y; \n\
    gl_Position.z = (gl_Position.z  + gl_Position.w) * 0.5; \n\
  #endif \n\
    vec2 colorTexCoord = mix(a_colorTexCoord, a_outlineColorTexCoord, isOutline); \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture(u_colorTex, colorTexCoord); \n\
  #else \n\
    v_colorTexCoord = colorTexCoord; \n\
  #endif \n\
    v_maskTexCoord = a_maskTexCoord; \n\
  } \n\
";

char const SMAA_BLENDING_WEIGHT_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform sampler2D u_colorTex; \n\
  uniform sampler2D u_smaaArea; \n\
  uniform sampler2D u_smaaSearch; \n\
  uniform vec4 u_framebufferMetrics; \n\
  varying vec4 v_coords; \n\
  varying vec4 v_offset0; \n\
  varying vec4 v_offset1; \n\
  varying vec4 v_offset2; \n\
  #define SMAA_SEARCHTEX_SIZE vec2(66.0, 33.0) \n\
  #define SMAA_SEARCHTEX_PACKED_SIZE vec2(64.0, 16.0) \n\
  #define SMAA_AREATEX_MAX_DISTANCE 16.0 \n\
  #define SMAA_AREATEX_PIXEL_SIZE (vec2(1.0 / 256.0, 1.0 / 1024.0)) \n\
  #ifdef GLES3 \n\
    #define SMAALoopBegin(condition) while (condition) { \n\
    #define SMAALoopEnd } \n\
    #define SMAASampleLevelZero(tex, coord) textureLod(tex, coord, 0.0) \n\
    #define SMAASampleLevelZeroOffset(tex, coord, offset) textureLodOffset(tex, coord, 0.0, offset) \n\
    #define SMAARound(v) round((v)) \n\
    #define SMAAOffset(x,y) ivec2(x,y) \n\
  #else \n\
    #define SMAA_MAX_SEARCH_STEPS 8 \n\
    #define SMAALoopBegin(condition) for (int i = 0; i < SMAA_MAX_SEARCH_STEPS; i++) { if (!(condition)) break; \n\
    #define SMAALoopEnd } \n\
    #define SMAASampleLevelZero(tex, coord) texture2D(tex, coord) \n\
    #define SMAASampleLevelZeroOffset(tex, coord, offset) texture2D(tex, coord + vec2(offset) * u_framebufferMetrics.xy) \n\
    #define SMAARound(v) floor((v) + 0.5) \n\
    #define SMAAOffset(x,y) vec2(x,y) \n\
  #endif \n\
  const vec2 kAreaTexMaxDistance = vec2(SMAA_AREATEX_MAX_DISTANCE, SMAA_AREATEX_MAX_DISTANCE); \n\
  const float kActivationThreshold = 0.8281; \n\
  float SMAASearchLength(vec2 e, float offset) \n\
  { \n\
    vec2 scale = SMAA_SEARCHTEX_SIZE * vec2(0.5, -1.0); \n\
    vec2 bias = SMAA_SEARCHTEX_SIZE * vec2(offset, 1.0); \n\
    scale += vec2(-1.0,  1.0); \n\
    bias += vec2( 0.5, -0.5); \n\
    scale *= 1.0 / SMAA_SEARCHTEX_PACKED_SIZE; \n\
    bias *= 1.0 / SMAA_SEARCHTEX_PACKED_SIZE; \n\
  #ifdef GLES3 \n\
    return SMAASampleLevelZero(u_smaaSearch, scale * e + bias).r; \n\
  #else \n\
    return SMAASampleLevelZero(u_smaaSearch, scale * e + bias).a; \n\
  #endif \n\
  } \n\
  float SMAASearchXLeft(vec2 texcoord, float end) \n\
  { \n\
    vec2 e = vec2(0.0, 1.0); \n\
    SMAALoopBegin(texcoord.x > end && e.g > kActivationThreshold && e.r == 0.0) \n\
      e = SMAASampleLevelZero(u_colorTex, texcoord).rg; \n\
      texcoord = vec2(-2.0, 0.0) * u_framebufferMetrics.xy + texcoord; \n\
    SMAALoopEnd \n\
    float offset = 3.25 - (255.0 / 127.0) * SMAASearchLength(e, 0.0); \n\
    return u_framebufferMetrics.x * offset + texcoord.x; \n\
  } \n\
  float SMAASearchXRight(vec2 texcoord, float end) \n\
  { \n\
    vec2 e = vec2(0.0, 1.0); \n\
    SMAALoopBegin(texcoord.x < end && e.g > kActivationThreshold && e.r == 0.0) \n\
      e = SMAASampleLevelZero(u_colorTex, texcoord).rg; \n\
      texcoord = vec2(2.0, 0.0) * u_framebufferMetrics.xy + texcoord; \n\
    SMAALoopEnd \n\
    float offset = 3.25 - (255.0 / 127.0) * SMAASearchLength(e, 0.5); \n\
    return -u_framebufferMetrics.x * offset + texcoord.x; \n\
  } \n\
  float SMAASearchYUp(vec2 texcoord, float end) \n\
  { \n\
    vec2 e = vec2(1.0, 0.0); \n\
    SMAALoopBegin(texcoord.y > end && e.r > kActivationThreshold && e.g == 0.0) \n\
      e = SMAASampleLevelZero(u_colorTex, texcoord).rg; \n\
      texcoord = vec2(0.0, -2.0) * u_framebufferMetrics.xy + texcoord; \n\
    SMAALoopEnd \n\
    float offset = 3.25 - (255.0 / 127.0) * SMAASearchLength(e.gr, 0.0); \n\
    return u_framebufferMetrics.y * offset + texcoord.y; \n\
  } \n\
  float SMAASearchYDown(vec2 texcoord, float end) \n\
  { \n\
    vec2 e = vec2(1.0, 0.0); \n\
    SMAALoopBegin(texcoord.y < end && e.r > kActivationThreshold && e.g == 0.0) \n\
      e = SMAASampleLevelZero(u_colorTex, texcoord).rg; \n\
      texcoord = vec2(0.0, 2.0) * u_framebufferMetrics.xy + texcoord; \n\
    SMAALoopEnd \n\
    float offset = 3.25 - (255.0 / 127.0) * SMAASearchLength(e.gr, 0.5); \n\
    return -u_framebufferMetrics.y * offset + texcoord.y; \n\
  } \n\
  vec2 SMAAArea(vec2 dist, float e1, float e2) \n\
  { \n\
    vec2 texcoord = kAreaTexMaxDistance * SMAARound(4.0 * vec2(e1, e2)) + dist; \n\
    texcoord = SMAA_AREATEX_PIXEL_SIZE * (texcoord + 0.5); \n\
    return SMAASampleLevelZero(u_smaaArea, texcoord).rg; \n\
  } \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
    vec4 weights = vec4(0.0, 0.0, 0.0, 0.0); \n\
    vec2 e = texture2D(u_colorTex, v_coords.xy).rg; \n\
    if (e.g > 0.0) // Edge at north \n\
    { \n\
      vec2 d; \n\
      vec3 coords; \n\
      coords.x = SMAASearchXLeft(v_offset0.xy, v_offset2.x); \n\
      coords.y = v_offset1.y; \n\
      d.x = coords.x; \n\
      float e1 = SMAASampleLevelZero(u_colorTex, coords.xy).r; \n\
      coords.z = SMAASearchXRight(v_offset0.zw, v_offset2.y); \n\
      d.y = coords.z; \n\
      vec2 zz = u_framebufferMetrics.zz; \n\
      d = abs(SMAARound(zz * d - v_coords.zz)); \n\
      vec2 sqrt_d = sqrt(d); \n\
      float e2 = SMAASampleLevelZeroOffset(u_colorTex, coords.zy, SMAAOffset(1, 0)).r; \n\
      weights.rg = SMAAArea(sqrt_d, e1, e2); \n\
    } \n\
    if (e.r > 0.0) // Edge at west \n\
    { \n\
      vec2 d; \n\
      vec3 coords; \n\
      coords.y = SMAASearchYUp(v_offset1.xy, v_offset2.z); \n\
      coords.x = v_offset0.x; \n\
      d.x = coords.y; \n\
      float e1 = SMAASampleLevelZero(u_colorTex, coords.xy).g; \n\
      coords.z = SMAASearchYDown(v_offset1.zw, v_offset2.w); \n\
      d.y = coords.z; \n\
      vec2 ww = u_framebufferMetrics.ww; \n\
      d = abs(SMAARound(ww * d - v_coords.ww)); \n\
      vec2 sqrt_d = sqrt(d); \n\
      float e2 = SMAASampleLevelZeroOffset(u_colorTex, coords.xz, SMAAOffset(0, 1)).g; \n\
      weights.ba = SMAAArea(sqrt_d, e1, e2); \n\
    } \n\
    gl_FragColor = weights; \n\
  } \n\
";

char const GLES3_SMAA_BLENDING_WEIGHT_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform sampler2D u_colorTex; \n\
  uniform sampler2D u_smaaArea; \n\
  uniform sampler2D u_smaaSearch; \n\
  uniform vec4 u_framebufferMetrics; \n\
  in vec4 v_coords; \n\
  in vec4 v_offset0; \n\
  in vec4 v_offset1; \n\
  in vec4 v_offset2; \n\
  #define SMAA_SEARCHTEX_SIZE vec2(66.0, 33.0) \n\
  #define SMAA_SEARCHTEX_PACKED_SIZE vec2(64.0, 16.0) \n\
  #define SMAA_AREATEX_MAX_DISTANCE 16.0 \n\
  #define SMAA_AREATEX_PIXEL_SIZE (vec2(1.0 / 256.0, 1.0 / 1024.0)) \n\
  #ifdef GLES3 \n\
    #define SMAALoopBegin(condition) while (condition) { \n\
    #define SMAALoopEnd } \n\
    #define SMAASampleLevelZero(tex, coord) textureLod(tex, coord, 0.0) \n\
    #define SMAASampleLevelZeroOffset(tex, coord, offset) textureLodOffset(tex, coord, 0.0, offset) \n\
    #define SMAARound(v) round((v)) \n\
    #define SMAAOffset(x,y) ivec2(x,y) \n\
  #else \n\
    #define SMAA_MAX_SEARCH_STEPS 8 \n\
    #define SMAALoopBegin(condition) for (int i = 0; i < SMAA_MAX_SEARCH_STEPS; i++) { if (!(condition)) break; \n\
    #define SMAALoopEnd } \n\
    #define SMAASampleLevelZero(tex, coord) texture(tex, coord) \n\
    #define SMAASampleLevelZeroOffset(tex, coord, offset) texture(tex, coord + vec2(offset) * u_framebufferMetrics.xy) \n\
    #define SMAARound(v) floor((v) + 0.5) \n\
    #define SMAAOffset(x,y) vec2(x,y) \n\
  #endif \n\
  const vec2 kAreaTexMaxDistance = vec2(SMAA_AREATEX_MAX_DISTANCE, SMAA_AREATEX_MAX_DISTANCE); \n\
  const float kActivationThreshold = 0.8281; \n\
  float SMAASearchLength(vec2 e, float offset) \n\
  { \n\
    vec2 scale = SMAA_SEARCHTEX_SIZE * vec2(0.5, -1.0); \n\
    vec2 bias = SMAA_SEARCHTEX_SIZE * vec2(offset, 1.0); \n\
    scale += vec2(-1.0,  1.0); \n\
    bias += vec2( 0.5, -0.5); \n\
    scale *= 1.0 / SMAA_SEARCHTEX_PACKED_SIZE; \n\
    bias *= 1.0 / SMAA_SEARCHTEX_PACKED_SIZE; \n\
  #ifdef GLES3 \n\
    return SMAASampleLevelZero(u_smaaSearch, scale * e + bias).r; \n\
  #else \n\
    return SMAASampleLevelZero(u_smaaSearch, scale * e + bias).a; \n\
  #endif \n\
  } \n\
  float SMAASearchXLeft(vec2 texcoord, float end) \n\
  { \n\
    vec2 e = vec2(0.0, 1.0); \n\
    SMAALoopBegin(texcoord.x > end && e.g > kActivationThreshold && e.r == 0.0) \n\
      e = SMAASampleLevelZero(u_colorTex, texcoord).rg; \n\
      texcoord = vec2(-2.0, 0.0) * u_framebufferMetrics.xy + texcoord; \n\
    SMAALoopEnd \n\
    float offset = 3.25 - (255.0 / 127.0) * SMAASearchLength(e, 0.0); \n\
    return u_framebufferMetrics.x * offset + texcoord.x; \n\
  } \n\
  float SMAASearchXRight(vec2 texcoord, float end) \n\
  { \n\
    vec2 e = vec2(0.0, 1.0); \n\
    SMAALoopBegin(texcoord.x < end && e.g > kActivationThreshold && e.r == 0.0) \n\
      e = SMAASampleLevelZero(u_colorTex, texcoord).rg; \n\
      texcoord = vec2(2.0, 0.0) * u_framebufferMetrics.xy + texcoord; \n\
    SMAALoopEnd \n\
    float offset = 3.25 - (255.0 / 127.0) * SMAASearchLength(e, 0.5); \n\
    return -u_framebufferMetrics.x * offset + texcoord.x; \n\
  } \n\
  float SMAASearchYUp(vec2 texcoord, float end) \n\
  { \n\
    vec2 e = vec2(1.0, 0.0); \n\
    SMAALoopBegin(texcoord.y > end && e.r > kActivationThreshold && e.g == 0.0) \n\
      e = SMAASampleLevelZero(u_colorTex, texcoord).rg; \n\
      texcoord = vec2(0.0, -2.0) * u_framebufferMetrics.xy + texcoord; \n\
    SMAALoopEnd \n\
    float offset = 3.25 - (255.0 / 127.0) * SMAASearchLength(e.gr, 0.0); \n\
    return u_framebufferMetrics.y * offset + texcoord.y; \n\
  } \n\
  float SMAASearchYDown(vec2 texcoord, float end) \n\
  { \n\
    vec2 e = vec2(1.0, 0.0); \n\
    SMAALoopBegin(texcoord.y < end && e.r > kActivationThreshold && e.g == 0.0) \n\
      e = SMAASampleLevelZero(u_colorTex, texcoord).rg; \n\
      texcoord = vec2(0.0, 2.0) * u_framebufferMetrics.xy + texcoord; \n\
    SMAALoopEnd \n\
    float offset = 3.25 - (255.0 / 127.0) * SMAASearchLength(e.gr, 0.5); \n\
    return -u_framebufferMetrics.y * offset + texcoord.y; \n\
  } \n\
  vec2 SMAAArea(vec2 dist, float e1, float e2) \n\
  { \n\
    vec2 texcoord = kAreaTexMaxDistance * SMAARound(4.0 * vec2(e1, e2)) + dist; \n\
    texcoord = SMAA_AREATEX_PIXEL_SIZE * (texcoord + 0.5); \n\
    return SMAASampleLevelZero(u_smaaArea, texcoord).rg; \n\
  } \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
    vec4 weights = vec4(0.0, 0.0, 0.0, 0.0); \n\
    vec2 e = texture(u_colorTex, v_coords.xy).rg; \n\
    if (e.g > 0.0) // Edge at north \n\
    { \n\
      vec2 d; \n\
      vec3 coords; \n\
      coords.x = SMAASearchXLeft(v_offset0.xy, v_offset2.x); \n\
      coords.y = v_offset1.y; \n\
      d.x = coords.x; \n\
      float e1 = SMAASampleLevelZero(u_colorTex, coords.xy).r; \n\
      coords.z = SMAASearchXRight(v_offset0.zw, v_offset2.y); \n\
      d.y = coords.z; \n\
      vec2 zz = u_framebufferMetrics.zz; \n\
      d = abs(SMAARound(zz * d - v_coords.zz)); \n\
      vec2 sqrt_d = sqrt(d); \n\
      float e2 = SMAASampleLevelZeroOffset(u_colorTex, coords.zy, SMAAOffset(1, 0)).r; \n\
      weights.rg = SMAAArea(sqrt_d, e1, e2); \n\
    } \n\
    if (e.r > 0.0) // Edge at west \n\
    { \n\
      vec2 d; \n\
      vec3 coords; \n\
      coords.y = SMAASearchYUp(v_offset1.xy, v_offset2.z); \n\
      coords.x = v_offset0.x; \n\
      d.x = coords.y; \n\
      float e1 = SMAASampleLevelZero(u_colorTex, coords.xy).g; \n\
      coords.z = SMAASearchYDown(v_offset1.zw, v_offset2.w); \n\
      d.y = coords.z; \n\
      vec2 ww = u_framebufferMetrics.ww; \n\
      d = abs(SMAARound(ww * d - v_coords.ww)); \n\
      vec2 sqrt_d = sqrt(d); \n\
      float e2 = SMAASampleLevelZeroOffset(u_colorTex, coords.xz, SMAAOffset(0, 1)).g; \n\
      weights.ba = SMAAArea(sqrt_d, e1, e2); \n\
    } \n\
    v_FragColor = weights; \n\
  } \n\
";

char const PATH_SYMBOL_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec4 a_position; \n\
  attribute vec2 a_normal; \n\
  attribute vec2 a_colorTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  varying vec2 v_colorTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pos = vec4(a_position.xyz, 1) * u_modelView; \n\
    float normalLen = length(a_normal); \n\
    vec4 n = vec4(a_position.xy + a_normal * kShapeCoordScalar, 0.0, 0.0) * u_modelView; \n\
    vec4 norm = vec4(0.0, 0.0, 0.0, 0.0); \n\
    if (dot(n, n) != 0.0) \n\
      norm = normalize(n) * normalLen; \n\
    vec4 shiftedPos = norm + pos; \n\
    gl_Position = applyPivotTransform(shiftedPos * u_projection, u_pivotTransform, 0.0); \n\
    v_colorTexCoords = a_colorTexCoords; \n\
  } \n\
";

char const GLES3_PATH_SYMBOL_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec4 a_position; \n\
  in vec2 a_normal; \n\
  in vec2 a_colorTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  out vec2 v_colorTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pos = vec4(a_position.xyz, 1) * u_modelView; \n\
    float normalLen = length(a_normal); \n\
    vec4 n = vec4(a_position.xy + a_normal * kShapeCoordScalar, 0.0, 0.0) * u_modelView; \n\
    vec4 norm = vec4(0.0, 0.0, 0.0, 0.0); \n\
    if (dot(n, n) != 0.0) \n\
      norm = normalize(n) * normalLen; \n\
    vec4 shiftedPos = norm + pos; \n\
    gl_Position = applyPivotTransform(shiftedPos * u_projection, u_pivotTransform, 0.0); \n\
    v_colorTexCoords = a_colorTexCoords; \n\
  } \n\
";

char const DEBUG_RECT_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform vec4 u_color; \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
  uniform sampler2D u_colorTex; \n\
  #endif \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
    gl_FragColor = samsungGoogleNexusWorkaround(u_color); \n\
  } \n\
";

char const GLES3_DEBUG_RECT_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform vec4 u_color; \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
  uniform sampler2D u_colorTex; \n\
  #endif \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
    v_FragColor = samsungGoogleNexusWorkaround(u_color); \n\
  } \n\
";

char const RULER_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec2 a_position; \n\
  attribute vec2 a_normal; \n\
  attribute vec2 a_colorTexCoords; \n\
  uniform vec2 u_position; \n\
  uniform float u_length; \n\
  uniform mat4 u_projection; \n\
  varying vec2 v_colorTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    gl_Position = vec4(u_position + a_position + u_length * a_normal, 0, 1) * u_projection; \n\
  #ifdef VULKAN \n\
    gl_Position.y = -gl_Position.y; \n\
    gl_Position.z = (gl_Position.z  + gl_Position.w) * 0.5; \n\
  #endif \n\
    v_colorTexCoords = a_colorTexCoords; \n\
  } \n\
";

char const GLES3_RULER_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec2 a_position; \n\
  in vec2 a_normal; \n\
  in vec2 a_colorTexCoords; \n\
  uniform vec2 u_position; \n\
  uniform float u_length; \n\
  uniform mat4 u_projection; \n\
  out vec2 v_colorTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    gl_Position = vec4(u_position + a_position + u_length * a_normal, 0, 1) * u_projection; \n\
  #ifdef VULKAN \n\
    gl_Position.y = -gl_Position.y; \n\
    gl_Position.z = (gl_Position.z  + gl_Position.w) * 0.5; \n\
  #endif \n\
    v_colorTexCoords = a_colorTexCoords; \n\
  } \n\
";

char const TRANSIT_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
  uniform sampler2D u_colorTex; \n\
  #endif \n\
  varying vec4 v_color; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
    gl_FragColor = samsungGoogleNexusWorkaround(v_color); \n\
  } \n\
";

char const GLES3_TRANSIT_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
  uniform sampler2D u_colorTex; \n\
  #endif \n\
  in vec4 v_color; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
    v_FragColor = samsungGoogleNexusWorkaround(v_color); \n\
  } \n\
";

char const TRAFFIC_CIRCLE_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec4 a_position; \n\
  attribute vec4 a_normal; \n\
  attribute vec2 a_colorTexCoord; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform vec3 u_lightArrowColor; // Here we store left sizes by road classes. \n\
  uniform vec3 u_darkArrowColor; // Here we store right sizes by road classes. \n\
  varying vec2 v_colorTexCoord; \n\
  varying vec3 v_radius; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec2 normal = a_normal.xy; \n\
    vec2 transformedAxisPos = (vec4(a_position.xy, 0.0, 1.0) * u_modelView).xy; \n\
    int index = int(a_position.w); \n\
    float leftSize = u_lightArrowColor[index]; \n\
    float rightSize = u_darkArrowColor[index]; \n\
    if (dot(normal, normal) != 0.0) \n\
    { \n\
      vec2 norm = normal * 0.5 * (rightSize - leftSize); \n\
      transformedAxisPos = calcLineTransformedAxisPos(transformedAxisPos, a_position.xy + norm, \n\
                                                      u_modelView, length(norm)); \n\
    } \n\
    v_radius = vec3(a_normal.zw, 1.0) * 0.5 * (leftSize + rightSize); \n\
    vec2 finalPos = transformedAxisPos + v_radius.xy; \n\
    v_colorTexCoord = a_colorTexCoord; \n\
    vec4 pos = vec4(finalPos, a_position.z, 1.0) * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
  } \n\
";

char const GLES3_TRAFFIC_CIRCLE_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec4 a_position; \n\
  in vec4 a_normal; \n\
  in vec2 a_colorTexCoord; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform vec3 u_lightArrowColor; // Here we store left sizes by road classes. \n\
  uniform vec3 u_darkArrowColor; // Here we store right sizes by road classes. \n\
  out vec2 v_colorTexCoord; \n\
  out vec3 v_radius; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec2 normal = a_normal.xy; \n\
    vec2 transformedAxisPos = (vec4(a_position.xy, 0.0, 1.0) * u_modelView).xy; \n\
    int index = int(a_position.w); \n\
    float leftSize = u_lightArrowColor[index]; \n\
    float rightSize = u_darkArrowColor[index]; \n\
    if (dot(normal, normal) != 0.0) \n\
    { \n\
      vec2 norm = normal * 0.5 * (rightSize - leftSize); \n\
      transformedAxisPos = calcLineTransformedAxisPos(transformedAxisPos, a_position.xy + norm, \n\
                                                      u_modelView, length(norm)); \n\
    } \n\
    v_radius = vec3(a_normal.zw, 1.0) * 0.5 * (leftSize + rightSize); \n\
    vec2 finalPos = transformedAxisPos + v_radius.xy; \n\
    v_colorTexCoord = a_colorTexCoord; \n\
    vec4 pos = vec4(finalPos, a_position.z, 1.0) * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
  } \n\
";

char const LINE_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform float u_opacity; \n\
  #ifdef ENABLE_VTF \n\
  varying LOW_P vec4 v_color; \n\
  #else \n\
  uniform sampler2D u_colorTex; \n\
  varying vec2 v_colorTexCoord; \n\
  #endif \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
  #ifdef ENABLE_VTF \n\
    LOW_P vec4 color = v_color; \n\
  #else \n\
    LOW_P vec4 color = texture2D(u_colorTex, v_colorTexCoord); \n\
  #endif \n\
    color.a *= u_opacity; \n\
    gl_FragColor = color; \n\
  } \n\
";

char const GLES3_LINE_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform float u_opacity; \n\
  #ifdef ENABLE_VTF \n\
  in LOW_P vec4 v_color; \n\
  #else \n\
  uniform sampler2D u_colorTex; \n\
  in vec2 v_colorTexCoord; \n\
  #endif \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
  #ifdef ENABLE_VTF \n\
    LOW_P vec4 color = v_color; \n\
  #else \n\
    LOW_P vec4 color = texture(u_colorTex, v_colorTexCoord); \n\
  #endif \n\
    color.a *= u_opacity; \n\
    v_FragColor = color; \n\
  } \n\
";

char const CIRCLE_POINT_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
  uniform sampler2D u_colorTex; \n\
  #endif \n\
  uniform float u_opacity; \n\
  varying vec3 v_radius; \n\
  varying vec4 v_color; \n\
  const float kAntialiasingScalar = 0.9; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
    float d = dot(v_radius.xy, v_radius.xy); \n\
    vec4 finalColor = v_color; \n\
   \n\
    float aaRadius = v_radius.z * kAntialiasingScalar; \n\
    float stepValue = smoothstep(aaRadius * aaRadius, v_radius.z * v_radius.z, d); \n\
    finalColor.a = finalColor.a * u_opacity * (1.0 - stepValue); \n\
    gl_FragColor = samsungGoogleNexusWorkaround(finalColor); \n\
  } \n\
";

char const GLES3_CIRCLE_POINT_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
  uniform sampler2D u_colorTex; \n\
  #endif \n\
  uniform float u_opacity; \n\
  in vec3 v_radius; \n\
  in vec4 v_color; \n\
  const float kAntialiasingScalar = 0.9; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
    float d = dot(v_radius.xy, v_radius.xy); \n\
    vec4 finalColor = v_color; \n\
   \n\
    float aaRadius = v_radius.z * kAntialiasingScalar; \n\
    float stepValue = smoothstep(aaRadius * aaRadius, v_radius.z * v_radius.z, d); \n\
    finalColor.a = finalColor.a * u_opacity * (1.0 - stepValue); \n\
    v_FragColor = samsungGoogleNexusWorkaround(finalColor); \n\
  } \n\
";

char const ARROW3D_SHADOW_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  varying float v_intensity; \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
  uniform sampler2D u_colorTex; \n\
  #endif \n\
  uniform vec4 u_color; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
    vec4 resColor = vec4(u_color.rgb, u_color.a * v_intensity); \n\
    gl_FragColor = samsungGoogleNexusWorkaround(resColor); \n\
  } \n\
";

char const GLES3_ARROW3D_SHADOW_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in float v_intensity; \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
  uniform sampler2D u_colorTex; \n\
  #endif \n\
  uniform vec4 u_color; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
    vec4 resColor = vec4(u_color.rgb, u_color.a * v_intensity); \n\
    v_FragColor = samsungGoogleNexusWorkaround(resColor); \n\
  } \n\
";

char const SELECTION_LINE_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  #ifdef ENABLE_VTF \n\
  varying LOW_P vec4 v_color; \n\
  #else \n\
  uniform sampler2D u_colorTex; \n\
  varying vec2 v_colorTexCoord; \n\
  #endif \n\
  uniform float u_opacity; \n\
  varying float v_lengthY; \n\
  const float kAntialiasingThreshold = 0.92; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
  #ifdef ENABLE_VTF \n\
    LOW_P vec4 color = v_color; \n\
  #else \n\
    LOW_P vec4 color = texture2D(u_colorTex, v_colorTexCoord); \n\
  #endif \n\
    color.a *= u_opacity; \n\
    color.a *= (1.0 - smoothstep(kAntialiasingThreshold, 1.0, abs(v_lengthY))); \n\
    gl_FragColor = samsungGoogleNexusWorkaround(color); \n\
  } \n\
";

char const GLES3_SELECTION_LINE_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  #ifdef ENABLE_VTF \n\
  in LOW_P vec4 v_color; \n\
  #else \n\
  uniform sampler2D u_colorTex; \n\
  in vec2 v_colorTexCoord; \n\
  #endif \n\
  uniform float u_opacity; \n\
  in float v_lengthY; \n\
  const float kAntialiasingThreshold = 0.92; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
  #ifdef ENABLE_VTF \n\
    LOW_P vec4 color = v_color; \n\
  #else \n\
    LOW_P vec4 color = texture(u_colorTex, v_colorTexCoord); \n\
  #endif \n\
    color.a *= u_opacity; \n\
    color.a *= (1.0 - smoothstep(kAntialiasingThreshold, 1.0, abs(v_lengthY))); \n\
    v_FragColor = samsungGoogleNexusWorkaround(color); \n\
  } \n\
";

char const USER_MARK_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform sampler2D u_colorTex; \n\
  uniform float u_opacity; \n\
  varying vec4 v_texCoords; \n\
  varying vec4 v_maskColor; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
    vec4 color = texture2D(u_colorTex, v_texCoords.xy); \n\
    vec4 bgColor = texture2D(u_colorTex, v_texCoords.zw) * vec4(v_maskColor.xyz, 1.0); \n\
    vec4 finalColor = mix(color, mix(bgColor, color, color.a), bgColor.a); \n\
    finalColor.a = clamp(color.a + bgColor.a, 0.0, 1.0) * u_opacity * v_maskColor.w; \n\
    if (finalColor.a < 0.01) \n\
      discard; \n\
    gl_FragColor = finalColor; \n\
  } \n\
";

char const GLES3_USER_MARK_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform sampler2D u_colorTex; \n\
  uniform float u_opacity; \n\
  in vec4 v_texCoords; \n\
  in vec4 v_maskColor; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
    vec4 color = texture(u_colorTex, v_texCoords.xy); \n\
    vec4 bgColor = texture(u_colorTex, v_texCoords.zw) * vec4(v_maskColor.xyz, 1.0); \n\
    vec4 finalColor = mix(color, mix(bgColor, color, color.a), bgColor.a); \n\
    finalColor.a = clamp(color.a + bgColor.a, 0.0, 1.0) * u_opacity * v_maskColor.w; \n\
    if (finalColor.a < 0.01) \n\
      discard; \n\
    v_FragColor = finalColor; \n\
  } \n\
";

char const DEBUG_RECT_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec2 a_position; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    gl_Position = vec4(a_position, 0, 1); \n\
  #ifdef VULKAN \n\
    gl_Position.y = -gl_Position.y; \n\
    gl_Position.z = (gl_Position.z  + gl_Position.w) * 0.5; \n\
  #endif \n\
  } \n\
";

char const GLES3_DEBUG_RECT_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec2 a_position; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    gl_Position = vec4(a_position, 0, 1); \n\
  #ifdef VULKAN \n\
    gl_Position.y = -gl_Position.y; \n\
    gl_Position.z = (gl_Position.z  + gl_Position.w) * 0.5; \n\
  #endif \n\
  } \n\
";

char const HATCHING_AREA_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec3 a_position; \n\
  attribute vec2 a_colorTexCoords; \n\
  attribute vec2 a_maskTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  varying LOW_P vec4 v_color; \n\
  #else \n\
  varying vec2 v_colorTexCoords; \n\
  #endif \n\
  varying vec2 v_maskTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pos = vec4(a_position, 1) * u_modelView * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture2D(u_colorTex, a_colorTexCoords); \n\
  #else \n\
    v_colorTexCoords = a_colorTexCoords; \n\
  #endif \n\
    v_maskTexCoords = a_maskTexCoords; \n\
  } \n\
";

char const GLES3_HATCHING_AREA_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 a_position; \n\
  in vec2 a_colorTexCoords; \n\
  in vec2 a_maskTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  out LOW_P vec4 v_color; \n\
  #else \n\
  out vec2 v_colorTexCoords; \n\
  #endif \n\
  out vec2 v_maskTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pos = vec4(a_position, 1) * u_modelView * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture(u_colorTex, a_colorTexCoords); \n\
  #else \n\
    v_colorTexCoords = a_colorTexCoords; \n\
  #endif \n\
    v_maskTexCoords = a_maskTexCoords; \n\
  } \n\
";

char const MY_POSITION_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec2 a_normal; \n\
  attribute vec2 a_colorTexCoords; \n\
  uniform vec3 u_position; \n\
  uniform float u_azimut; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  varying vec2 v_colorTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    float sinV = sin(u_azimut); \n\
    float cosV = cos(u_azimut); \n\
    mat4 rotation; \n\
    rotation[0] = vec4(cosV, sinV, 0.0, 0.0); \n\
    rotation[1] = vec4(-sinV, cosV, 0.0, 0.0); \n\
    rotation[2] = vec4(0.0, 0.0, 1.0, 0.0); \n\
    rotation[3] = vec4(0.0, 0.0, 0.0, 1.0); \n\
    vec4 pos = vec4(u_position, 1.0) * u_modelView; \n\
    vec4 normal = vec4(a_normal, 0, 0); \n\
    vec4 shiftedPos = normal * rotation + pos; \n\
    gl_Position = applyPivotTransform(shiftedPos * u_projection, u_pivotTransform, 0.0); \n\
    v_colorTexCoords = a_colorTexCoords; \n\
  } \n\
";

char const GLES3_MY_POSITION_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec2 a_normal; \n\
  in vec2 a_colorTexCoords; \n\
  uniform vec3 u_position; \n\
  uniform float u_azimut; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  out vec2 v_colorTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    float sinV = sin(u_azimut); \n\
    float cosV = cos(u_azimut); \n\
    mat4 rotation; \n\
    rotation[0] = vec4(cosV, sinV, 0.0, 0.0); \n\
    rotation[1] = vec4(-sinV, cosV, 0.0, 0.0); \n\
    rotation[2] = vec4(0.0, 0.0, 1.0, 0.0); \n\
    rotation[3] = vec4(0.0, 0.0, 0.0, 1.0); \n\
    vec4 pos = vec4(u_position, 1.0) * u_modelView; \n\
    vec4 normal = vec4(a_normal, 0, 0); \n\
    vec4 shiftedPos = normal * rotation + pos; \n\
    gl_Position = applyPivotTransform(shiftedPos * u_projection, u_pivotTransform, 0.0); \n\
    v_colorTexCoords = a_colorTexCoords; \n\
  } \n\
";

char const TRAFFIC_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  varying vec2 v_colorTexCoord; \n\
  varying vec2 v_maskTexCoord; \n\
  varying float v_halfLength; \n\
  uniform sampler2D u_colorTex; \n\
  uniform sampler2D u_maskTex; \n\
  uniform float u_opacity; \n\
  uniform float u_outline; \n\
  uniform vec3 u_lightArrowColor; \n\
  uniform vec3 u_darkArrowColor; \n\
  uniform vec3 u_outlineColor; \n\
  const float kAntialiasingThreshold = 0.92; \n\
  const float kOutlineThreshold1 = 0.8; \n\
  const float kOutlineThreshold2 = 0.5; \n\
  const float kMaskOpacity = 0.7; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
    vec4 color = texture2D(u_colorTex, v_colorTexCoord); \n\
    float alphaCode = color.a; \n\
    vec4 mask = texture2D(u_maskTex, v_maskTexCoord); \n\
    color.a = u_opacity * (1.0 - smoothstep(kAntialiasingThreshold, 1.0, abs(v_halfLength))); \n\
    color.rgb = mix(color.rgb, mask.rgb * mix(u_lightArrowColor, u_darkArrowColor, step(alphaCode, 0.6)), mask.a * kMaskOpacity); \n\
    if (u_outline > 0.0) \n\
    { \n\
      color.rgb = mix(color.rgb, u_outlineColor, step(kOutlineThreshold1, abs(v_halfLength))); \n\
      color.rgb = mix(color.rgb, u_outlineColor, smoothstep(kOutlineThreshold2, kOutlineThreshold1, abs(v_halfLength))); \n\
    } \n\
    gl_FragColor = color; \n\
  } \n\
";

char const GLES3_TRAFFIC_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec2 v_colorTexCoord; \n\
  in vec2 v_maskTexCoord; \n\
  in float v_halfLength; \n\
  uniform sampler2D u_colorTex; \n\
  uniform sampler2D u_maskTex; \n\
  uniform float u_opacity; \n\
  uniform float u_outline; \n\
  uniform vec3 u_lightArrowColor; \n\
  uniform vec3 u_darkArrowColor; \n\
  uniform vec3 u_outlineColor; \n\
  const float kAntialiasingThreshold = 0.92; \n\
  const float kOutlineThreshold1 = 0.8; \n\
  const float kOutlineThreshold2 = 0.5; \n\
  const float kMaskOpacity = 0.7; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
    vec4 color = texture(u_colorTex, v_colorTexCoord); \n\
    float alphaCode = color.a; \n\
    vec4 mask = texture(u_maskTex, v_maskTexCoord); \n\
    color.a = u_opacity * (1.0 - smoothstep(kAntialiasingThreshold, 1.0, abs(v_halfLength))); \n\
    color.rgb = mix(color.rgb, mask.rgb * mix(u_lightArrowColor, u_darkArrowColor, step(alphaCode, 0.6)), mask.a * kMaskOpacity); \n\
    if (u_outline > 0.0) \n\
    { \n\
      color.rgb = mix(color.rgb, u_outlineColor, step(kOutlineThreshold1, abs(v_halfLength))); \n\
      color.rgb = mix(color.rgb, u_outlineColor, smoothstep(kOutlineThreshold2, kOutlineThreshold1, abs(v_halfLength))); \n\
    } \n\
    v_FragColor = color; \n\
  } \n\
";

char const TRAFFIC_CIRCLE_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  varying vec2 v_colorTexCoord; \n\
  varying vec3 v_radius; \n\
  uniform sampler2D u_colorTex; \n\
  uniform float u_opacity; \n\
  const float kAntialiasingThreshold = 0.92; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
    vec4 color = texture2D(u_colorTex, v_colorTexCoord); \n\
    float smallRadius = v_radius.z * kAntialiasingThreshold; \n\
    float stepValue = smoothstep(smallRadius * smallRadius, v_radius.z * v_radius.z, \n\
                                 v_radius.x * v_radius.x + v_radius.y * v_radius.y); \n\
    color.a = u_opacity * (1.0 - stepValue); \n\
    if (color.a < 0.01) \n\
      discard; \n\
    gl_FragColor = color; \n\
  } \n\
";

char const GLES3_TRAFFIC_CIRCLE_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec2 v_colorTexCoord; \n\
  in vec3 v_radius; \n\
  uniform sampler2D u_colorTex; \n\
  uniform float u_opacity; \n\
  const float kAntialiasingThreshold = 0.92; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
    vec4 color = texture(u_colorTex, v_colorTexCoord); \n\
    float smallRadius = v_radius.z * kAntialiasingThreshold; \n\
    float stepValue = smoothstep(smallRadius * smallRadius, v_radius.z * v_radius.z, \n\
                                 v_radius.x * v_radius.x + v_radius.y * v_radius.y); \n\
    color.a = u_opacity * (1.0 - stepValue); \n\
    if (color.a < 0.01) \n\
      discard; \n\
    v_FragColor = color; \n\
  } \n\
";

char const COLORED_SYMBOL_BILLBOARD_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec3 a_position; \n\
  attribute vec4 a_normal; \n\
  attribute vec4 a_colorTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  varying vec4 v_normal; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  varying LOW_P vec4 v_color; \n\
  #else \n\
  varying vec2 v_colorTexCoords; \n\
  #endif \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pivot = vec4(a_position.xyz, 1.0) * u_modelView; \n\
    vec4 offset = vec4(a_normal.xy + a_colorTexCoords.zw, 0.0, 0.0) * u_projection; \n\
    gl_Position = applyBillboardPivotTransform(pivot * u_projection, u_pivotTransform, 0.0, offset.xy); \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture2D(u_colorTex, a_colorTexCoords.xy); \n\
  #else \n\
    v_colorTexCoords = a_colorTexCoords.xy; \n\
  #endif \n\
    v_normal = a_normal; \n\
  } \n\
";

char const GLES3_COLORED_SYMBOL_BILLBOARD_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 a_position; \n\
  in vec4 a_normal; \n\
  in vec4 a_colorTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  out vec4 v_normal; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  out LOW_P vec4 v_color; \n\
  #else \n\
  out vec2 v_colorTexCoords; \n\
  #endif \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pivot = vec4(a_position.xyz, 1.0) * u_modelView; \n\
    vec4 offset = vec4(a_normal.xy + a_colorTexCoords.zw, 0.0, 0.0) * u_projection; \n\
    gl_Position = applyBillboardPivotTransform(pivot * u_projection, u_pivotTransform, 0.0, offset.xy); \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture(u_colorTex, a_colorTexCoords.xy); \n\
  #else \n\
    v_colorTexCoords = a_colorTexCoords.xy; \n\
  #endif \n\
    v_normal = a_normal; \n\
  } \n\
";

char const TRANSIT_MARKER_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
  uniform sampler2D u_colorTex; \n\
  #endif \n\
  varying vec4 v_offsets; \n\
  varying vec4 v_color; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
    vec4 finalColor = v_color; \n\
    vec2 radius; \n\
    radius.x = max(0.0, abs(v_offsets.x) - v_offsets.z); \n\
    radius.y = max(0.0, abs(v_offsets.y) - v_offsets.w); \n\
    float maxRadius = 1.0; \n\
    float aaRadius = 0.9; \n\
    float stepValue = smoothstep(aaRadius * aaRadius, maxRadius * maxRadius, dot(radius.xy, radius.xy)); \n\
    finalColor.a = finalColor.a * (1.0 - stepValue); \n\
    gl_FragColor = samsungGoogleNexusWorkaround(finalColor); \n\
  } \n\
";

char const GLES3_TRANSIT_MARKER_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
  uniform sampler2D u_colorTex; \n\
  #endif \n\
  in vec4 v_offsets; \n\
  in vec4 v_color; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
    vec4 finalColor = v_color; \n\
    vec2 radius; \n\
    radius.x = max(0.0, abs(v_offsets.x) - v_offsets.z); \n\
    radius.y = max(0.0, abs(v_offsets.y) - v_offsets.w); \n\
    float maxRadius = 1.0; \n\
    float aaRadius = 0.9; \n\
    float stepValue = smoothstep(aaRadius * aaRadius, maxRadius * maxRadius, dot(radius.xy, radius.xy)); \n\
    finalColor.a = finalColor.a * (1.0 - stepValue); \n\
    v_FragColor = samsungGoogleNexusWorkaround(finalColor); \n\
  } \n\
";

char const TRANSIT_MARKER_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec3 a_position; \n\
  attribute vec4 a_normal; \n\
  attribute vec4 a_color; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform vec3 u_params; \n\
  varying vec4 v_offsets; \n\
  varying vec4 v_color; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pos = vec4(a_position.xy, 0, 1) * u_modelView; \n\
    vec2 normal = vec2(a_normal.x * u_params.x - a_normal.y * u_params.y, \n\
                       a_normal.x * u_params.y + a_normal.y * u_params.x); \n\
    vec2 shiftedPos = normal * u_params.z + pos.xy; \n\
    pos = vec4(shiftedPos, a_position.z, 1.0) * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
    vec2 offsets = abs(a_normal.zw); \n\
    v_offsets = vec4(a_normal.zw, offsets - 1.0); \n\
    v_color = a_color; \n\
  } \n\
";

char const GLES3_TRANSIT_MARKER_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 a_position; \n\
  in vec4 a_normal; \n\
  in vec4 a_color; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform vec3 u_params; \n\
  out vec4 v_offsets; \n\
  out vec4 v_color; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pos = vec4(a_position.xy, 0, 1) * u_modelView; \n\
    vec2 normal = vec2(a_normal.x * u_params.x - a_normal.y * u_params.y, \n\
                       a_normal.x * u_params.y + a_normal.y * u_params.x); \n\
    vec2 shiftedPos = normal * u_params.z + pos.xy; \n\
    pos = vec4(shiftedPos, a_position.z, 1.0) * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
    vec2 offsets = abs(a_normal.zw); \n\
    v_offsets = vec4(a_normal.zw, offsets - 1.0); \n\
    v_color = a_color; \n\
  } \n\
";

char const SOLID_COLOR_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform float u_opacity; \n\
  #ifdef ENABLE_VTF \n\
  varying LOW_P vec4 v_color; \n\
  #else \n\
  uniform sampler2D u_colorTex; \n\
  varying vec2 v_colorTexCoords; \n\
  #endif \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
  #ifdef ENABLE_VTF \n\
    LOW_P vec4 finalColor = v_color; \n\
  #else \n\
    LOW_P vec4 finalColor = texture2D(u_colorTex, v_colorTexCoords); \n\
  #endif \n\
    finalColor.a *= u_opacity; \n\
    gl_FragColor = finalColor; \n\
  } \n\
";

char const GLES3_SOLID_COLOR_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform float u_opacity; \n\
  #ifdef ENABLE_VTF \n\
  in LOW_P vec4 v_color; \n\
  #else \n\
  uniform sampler2D u_colorTex; \n\
  in vec2 v_colorTexCoords; \n\
  #endif \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
  #ifdef ENABLE_VTF \n\
    LOW_P vec4 finalColor = v_color; \n\
  #else \n\
    LOW_P vec4 finalColor = texture(u_colorTex, v_colorTexCoords); \n\
  #endif \n\
    finalColor.a *= u_opacity; \n\
    v_FragColor = finalColor; \n\
  } \n\
";

char const CIRCLE_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform float u_opacity; \n\
  #ifdef ENABLE_VTF \n\
  varying LOW_P vec4 v_color; \n\
  #else \n\
  uniform sampler2D u_colorTex; \n\
  varying vec2 v_colorTexCoords; \n\
  #endif \n\
  varying vec3 v_radius; \n\
  const float aaPixelsCount = 2.5; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
  #ifdef ENABLE_VTF \n\
    LOW_P vec4 finalColor = v_color; \n\
  #else \n\
    LOW_P vec4 finalColor = texture2D(u_colorTex, v_colorTexCoords); \n\
  #endif \n\
    float smallRadius = v_radius.z - aaPixelsCount; \n\
    float stepValue = smoothstep(smallRadius * smallRadius, v_radius.z * v_radius.z, \n\
                                 v_radius.x * v_radius.x + v_radius.y * v_radius.y); \n\
    finalColor.a = finalColor.a * u_opacity * (1.0 - stepValue); \n\
    gl_FragColor = finalColor; \n\
  } \n\
";

char const GLES3_CIRCLE_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform float u_opacity; \n\
  #ifdef ENABLE_VTF \n\
  in LOW_P vec4 v_color; \n\
  #else \n\
  uniform sampler2D u_colorTex; \n\
  in vec2 v_colorTexCoords; \n\
  #endif \n\
  in vec3 v_radius; \n\
  const float aaPixelsCount = 2.5; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
  #ifdef ENABLE_VTF \n\
    LOW_P vec4 finalColor = v_color; \n\
  #else \n\
    LOW_P vec4 finalColor = texture(u_colorTex, v_colorTexCoords); \n\
  #endif \n\
    float smallRadius = v_radius.z - aaPixelsCount; \n\
    float stepValue = smoothstep(smallRadius * smallRadius, v_radius.z * v_radius.z, \n\
                                 v_radius.x * v_radius.x + v_radius.y * v_radius.y); \n\
    finalColor.a = finalColor.a * u_opacity * (1.0 - stepValue); \n\
    v_FragColor = finalColor; \n\
  } \n\
";

char const AREA3D_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec3 a_position; \n\
  attribute vec3 a_normal; \n\
  attribute vec2 a_colorTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_zScale; \n\
  varying vec2 v_colorTexCoords; \n\
  varying float v_intensity; \n\
  const vec4 kNormalizedLightDir = vec4(0.3162, 0.0, 0.9486, 0.0); \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pos = vec4(a_position, 1.0) * u_modelView; \n\
   \n\
    vec4 normal = vec4(a_position + a_normal, 1.0) * u_modelView; \n\
    normal.xyw = (normal * u_projection).xyw; \n\
    normal.z = normal.z * u_zScale; \n\
    pos.xyw = (pos * u_projection).xyw; \n\
    pos.z = a_position.z * u_zScale; \n\
    vec4 normDir = normal - pos; \n\
    if (dot(normDir, normDir) != 0.0) \n\
      v_intensity = max(0.0, -dot(kNormalizedLightDir, normalize(normDir))); \n\
    else \n\
      v_intensity = 0.0; \n\
    gl_Position = u_pivotTransform * pos; \n\
  #ifdef VULKAN \n\
    gl_Position.y = -gl_Position.y; \n\
    gl_Position.z = (gl_Position.z  + gl_Position.w) * 0.5; \n\
  #endif \n\
    v_colorTexCoords = a_colorTexCoords; \n\
  } \n\
";

char const GLES3_AREA3D_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 a_position; \n\
  in vec3 a_normal; \n\
  in vec2 a_colorTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_zScale; \n\
  out vec2 v_colorTexCoords; \n\
  out float v_intensity; \n\
  const vec4 kNormalizedLightDir = vec4(0.3162, 0.0, 0.9486, 0.0); \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pos = vec4(a_position, 1.0) * u_modelView; \n\
   \n\
    vec4 normal = vec4(a_position + a_normal, 1.0) * u_modelView; \n\
    normal.xyw = (normal * u_projection).xyw; \n\
    normal.z = normal.z * u_zScale; \n\
    pos.xyw = (pos * u_projection).xyw; \n\
    pos.z = a_position.z * u_zScale; \n\
    vec4 normDir = normal - pos; \n\
    if (dot(normDir, normDir) != 0.0) \n\
      v_intensity = max(0.0, -dot(kNormalizedLightDir, normalize(normDir))); \n\
    else \n\
      v_intensity = 0.0; \n\
    gl_Position = u_pivotTransform * pos; \n\
  #ifdef VULKAN \n\
    gl_Position.y = -gl_Position.y; \n\
    gl_Position.z = (gl_Position.z  + gl_Position.w) * 0.5; \n\
  #endif \n\
    v_colorTexCoords = a_colorTexCoords; \n\
  } \n\
";

char const USER_MARK_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec3 a_position; \n\
  attribute vec3 a_normalAndAnimateOrZ; \n\
  attribute vec4 a_texCoords; \n\
  attribute vec4 a_color; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_interpolation; \n\
  varying vec4 v_texCoords; \n\
  varying vec4 v_maskColor; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec2 normal = a_normalAndAnimateOrZ.xy; \n\
    if (a_normalAndAnimateOrZ.z > 0.0) \n\
      normal = u_interpolation * normal; \n\
    vec4 p = vec4(a_position, 1.0) * u_modelView; \n\
    vec4 pos = vec4(normal, 0.0, 0.0) + p; \n\
    vec4 projectedPivot = p * u_projection; \n\
    gl_Position = applyPivotTransform(pos * u_projection, u_pivotTransform, 0.0); \n\
    float newZ = projectedPivot.y / projectedPivot.w * 0.5 + 0.5; \n\
    gl_Position.z = abs(a_normalAndAnimateOrZ.z) * newZ + (1.0 - abs(a_normalAndAnimateOrZ.z)) * gl_Position.z; \n\
    v_texCoords = a_texCoords; \n\
    v_maskColor = a_color; \n\
  } \n\
";

char const GLES3_USER_MARK_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 a_position; \n\
  in vec3 a_normalAndAnimateOrZ; \n\
  in vec4 a_texCoords; \n\
  in vec4 a_color; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_interpolation; \n\
  out vec4 v_texCoords; \n\
  out vec4 v_maskColor; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec2 normal = a_normalAndAnimateOrZ.xy; \n\
    if (a_normalAndAnimateOrZ.z > 0.0) \n\
      normal = u_interpolation * normal; \n\
    vec4 p = vec4(a_position, 1.0) * u_modelView; \n\
    vec4 pos = vec4(normal, 0.0, 0.0) + p; \n\
    vec4 projectedPivot = p * u_projection; \n\
    gl_Position = applyPivotTransform(pos * u_projection, u_pivotTransform, 0.0); \n\
    float newZ = projectedPivot.y / projectedPivot.w * 0.5 + 0.5; \n\
    gl_Position.z = abs(a_normalAndAnimateOrZ.z) * newZ + (1.0 - abs(a_normalAndAnimateOrZ.z)) * gl_Position.z; \n\
    v_texCoords = a_texCoords; \n\
    v_maskColor = a_color; \n\
  } \n\
";

char const COLORED_SYMBOL_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform float u_opacity; \n\
  varying vec4 v_normal; \n\
  #ifdef ENABLE_VTF \n\
  varying LOW_P vec4 v_color; \n\
  #else \n\
  uniform sampler2D u_colorTex; \n\
  varying vec2 v_colorTexCoords; \n\
  #endif \n\
  const float aaPixelsCount = 2.5; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
  #ifdef ENABLE_VTF \n\
    LOW_P vec4 color = v_color; \n\
  #else \n\
    LOW_P vec4 color = texture2D(u_colorTex, v_colorTexCoords); \n\
  #endif \n\
    float r1 = (v_normal.z - aaPixelsCount) * (v_normal.z - aaPixelsCount); \n\
    float r2 = v_normal.x * v_normal.x + v_normal.y * v_normal.y; \n\
    float r3 = v_normal.z * v_normal.z; \n\
    float alpha = mix(step(r3, r2), smoothstep(r1, r3, r2), v_normal.w); \n\
    LOW_P vec4 finalColor = color; \n\
    finalColor.a = finalColor.a * u_opacity * (1.0 - alpha); \n\
    if (finalColor.a == 0.0) \n\
      discard; \n\
    gl_FragColor = finalColor; \n\
  } \n\
";

char const GLES3_COLORED_SYMBOL_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform float u_opacity; \n\
  in vec4 v_normal; \n\
  #ifdef ENABLE_VTF \n\
  in LOW_P vec4 v_color; \n\
  #else \n\
  uniform sampler2D u_colorTex; \n\
  in vec2 v_colorTexCoords; \n\
  #endif \n\
  const float aaPixelsCount = 2.5; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
  #ifdef ENABLE_VTF \n\
    LOW_P vec4 color = v_color; \n\
  #else \n\
    LOW_P vec4 color = texture(u_colorTex, v_colorTexCoords); \n\
  #endif \n\
    float r1 = (v_normal.z - aaPixelsCount) * (v_normal.z - aaPixelsCount); \n\
    float r2 = v_normal.x * v_normal.x + v_normal.y * v_normal.y; \n\
    float r3 = v_normal.z * v_normal.z; \n\
    float alpha = mix(step(r3, r2), smoothstep(r1, r3, r2), v_normal.w); \n\
    LOW_P vec4 finalColor = color; \n\
    finalColor.a = finalColor.a * u_opacity * (1.0 - alpha); \n\
    if (finalColor.a == 0.0) \n\
      discard; \n\
    v_FragColor = finalColor; \n\
  } \n\
";

char const MASKED_TEXTURING_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform sampler2D u_colorTex; \n\
  uniform sampler2D u_maskTex; \n\
  uniform float u_opacity; \n\
  varying vec2 v_colorTexCoords; \n\
  varying vec2 v_maskTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
    vec4 finalColor = texture2D(u_colorTex, v_colorTexCoords) * texture2D(u_maskTex, v_maskTexCoords); \n\
    finalColor.a *= u_opacity; \n\
    gl_FragColor = finalColor; \n\
  } \n\
";

char const GLES3_MASKED_TEXTURING_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform sampler2D u_colorTex; \n\
  uniform sampler2D u_maskTex; \n\
  uniform float u_opacity; \n\
  in vec2 v_colorTexCoords; \n\
  in vec2 v_maskTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
    vec4 finalColor = texture(u_colorTex, v_colorTexCoords) * texture(u_maskTex, v_maskTexCoords); \n\
    finalColor.a *= u_opacity; \n\
    v_FragColor = finalColor; \n\
  } \n\
";

char const ROUTE_MARKER_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
  uniform sampler2D u_colorTex; \n\
  #endif \n\
  uniform vec4 u_routeParams; \n\
  uniform vec4 u_maskColor; \n\
  uniform float u_opacity; \n\
  varying vec4 v_radius; \n\
  varying vec4 v_color; \n\
  const float kAntialiasingPixelsCount = 2.5; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
    vec4 finalColor = v_color; \n\
   \n\
    float aaRadius = max(v_radius.z - kAntialiasingPixelsCount, 0.0); \n\
    float stepValue = smoothstep(aaRadius * aaRadius, v_radius.z * v_radius.z, \n\
                                 dot(v_radius.xy, v_radius.xy)); \n\
    finalColor.a = finalColor.a * u_opacity * (1.0 - stepValue); \n\
    if (finalColor.a < 0.01 || u_routeParams.y > v_radius.w) \n\
      discard; \n\
    finalColor = vec4(mix(finalColor.rgb, u_maskColor.rgb, u_maskColor.a), finalColor.a); \n\
    gl_FragColor = samsungGoogleNexusWorkaround(finalColor); \n\
  } \n\
";

char const GLES3_ROUTE_MARKER_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
  uniform sampler2D u_colorTex; \n\
  #endif \n\
  uniform vec4 u_routeParams; \n\
  uniform vec4 u_maskColor; \n\
  uniform float u_opacity; \n\
  in vec4 v_radius; \n\
  in vec4 v_color; \n\
  const float kAntialiasingPixelsCount = 2.5; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
    vec4 finalColor = v_color; \n\
   \n\
    float aaRadius = max(v_radius.z - kAntialiasingPixelsCount, 0.0); \n\
    float stepValue = smoothstep(aaRadius * aaRadius, v_radius.z * v_radius.z, \n\
                                 dot(v_radius.xy, v_radius.xy)); \n\
    finalColor.a = finalColor.a * u_opacity * (1.0 - stepValue); \n\
    if (finalColor.a < 0.01 || u_routeParams.y > v_radius.w) \n\
      discard; \n\
    finalColor = vec4(mix(finalColor.rgb, u_maskColor.rgb, u_maskColor.a), finalColor.a); \n\
    v_FragColor = samsungGoogleNexusWorkaround(finalColor); \n\
  } \n\
";

char const TEXTURING_BILLBOARD_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec4 a_position; \n\
  attribute vec2 a_normal; \n\
  attribute vec2 a_colorTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_zScale; \n\
  varying vec2 v_colorTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pivot = vec4(a_position.xyz, 1.0) * u_modelView; \n\
    vec4 offset = vec4(a_normal, 0.0, 0.0) * u_projection; \n\
    gl_Position = applyBillboardPivotTransform(pivot * u_projection, u_pivotTransform, \n\
                                               a_position.w * u_zScale, offset.xy); \n\
    v_colorTexCoords = a_colorTexCoords; \n\
  } \n\
";

char const GLES3_TEXTURING_BILLBOARD_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec4 a_position; \n\
  in vec2 a_normal; \n\
  in vec2 a_colorTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_zScale; \n\
  out vec2 v_colorTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pivot = vec4(a_position.xyz, 1.0) * u_modelView; \n\
    vec4 offset = vec4(a_normal, 0.0, 0.0) * u_projection; \n\
    gl_Position = applyBillboardPivotTransform(pivot * u_projection, u_pivotTransform, \n\
                                               a_position.w * u_zScale, offset.xy); \n\
    v_colorTexCoords = a_colorTexCoords; \n\
  } \n\
";

char const TEXTURING_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform sampler2D u_colorTex; \n\
  uniform float u_opacity; \n\
  varying vec2 v_colorTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
    vec4 finalColor = texture2D(u_colorTex, v_colorTexCoords); \n\
    finalColor.a *= u_opacity; \n\
    gl_FragColor = finalColor; \n\
  } \n\
";

char const GLES3_TEXTURING_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform sampler2D u_colorTex; \n\
  uniform float u_opacity; \n\
  in vec2 v_colorTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
    vec4 finalColor = texture(u_colorTex, v_colorTexCoords); \n\
    finalColor.a *= u_opacity; \n\
    v_FragColor = finalColor; \n\
  } \n\
";

char const LINE_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec3 a_position; \n\
  attribute vec3 a_normal; \n\
  attribute vec2 a_colorTexCoord; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  varying LOW_P vec4 v_color; \n\
  #else \n\
  varying vec2 v_colorTexCoord; \n\
  #endif \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec2 normal = a_normal.xy; \n\
    float halfWidth = length(normal); \n\
    vec2 transformedAxisPos = (vec4(a_position.xy, 0.0, 1.0) * u_modelView).xy; \n\
    if (halfWidth != 0.0) \n\
    { \n\
      transformedAxisPos = calcLineTransformedAxisPos(transformedAxisPos, a_position.xy + normal, \n\
                                                      u_modelView, halfWidth); \n\
    } \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture2D(u_colorTex, a_colorTexCoord); \n\
  #else \n\
    v_colorTexCoord = a_colorTexCoord; \n\
  #endif \n\
    vec4 pos = vec4(transformedAxisPos, a_position.z, 1.0) * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
  } \n\
";

char const GLES3_LINE_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 a_position; \n\
  in vec3 a_normal; \n\
  in vec2 a_colorTexCoord; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  out LOW_P vec4 v_color; \n\
  #else \n\
  out vec2 v_colorTexCoord; \n\
  #endif \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec2 normal = a_normal.xy; \n\
    float halfWidth = length(normal); \n\
    vec2 transformedAxisPos = (vec4(a_position.xy, 0.0, 1.0) * u_modelView).xy; \n\
    if (halfWidth != 0.0) \n\
    { \n\
      transformedAxisPos = calcLineTransformedAxisPos(transformedAxisPos, a_position.xy + normal, \n\
                                                      u_modelView, halfWidth); \n\
    } \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture(u_colorTex, a_colorTexCoord); \n\
  #else \n\
    v_colorTexCoord = a_colorTexCoord; \n\
  #endif \n\
    vec4 pos = vec4(transformedAxisPos, a_position.z, 1.0) * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
  } \n\
";

char const TRAFFIC_LINE_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform sampler2D u_colorTex; \n\
  uniform float u_opacity; \n\
  varying vec2 v_colorTexCoord; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
    vec4 color = texture2D(u_colorTex, v_colorTexCoord); \n\
    gl_FragColor = vec4(color.rgb, u_opacity); \n\
  } \n\
";

char const GLES3_TRAFFIC_LINE_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform sampler2D u_colorTex; \n\
  uniform float u_opacity; \n\
  in vec2 v_colorTexCoord; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
    vec4 color = texture(u_colorTex, v_colorTexCoord); \n\
    v_FragColor = vec4(color.rgb, u_opacity); \n\
  } \n\
";

char const ROUTE_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec3 a_position; \n\
  attribute vec2 a_normal; \n\
  attribute vec3 a_length; \n\
  attribute vec4 a_color; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform vec4 u_routeParams; \n\
  varying vec3 v_length; \n\
  varying vec4 v_color; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec2 transformedAxisPos = (vec4(a_position.xy, 0.0, 1.0) * u_modelView).xy; \n\
    vec2 len = vec2(a_length.x, a_length.z); \n\
    if (dot(a_normal, a_normal) != 0.0) \n\
    { \n\
      vec2 norm = a_normal * u_routeParams.x; \n\
      transformedAxisPos = calcLineTransformedAxisPos(transformedAxisPos, a_position.xy + norm, \n\
                                                      u_modelView, length(norm)); \n\
      if (u_routeParams.y != 0.0) \n\
        len = vec2(a_length.x + a_length.y * u_routeParams.y, a_length.z); \n\
    } \n\
    v_length = vec3(len, u_routeParams.z); \n\
    v_color = a_color; \n\
    vec4 pos = vec4(transformedAxisPos, a_position.z, 1.0) * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
  } \n\
";

char const GLES3_ROUTE_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 a_position; \n\
  in vec2 a_normal; \n\
  in vec3 a_length; \n\
  in vec4 a_color; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform vec4 u_routeParams; \n\
  out vec3 v_length; \n\
  out vec4 v_color; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec2 transformedAxisPos = (vec4(a_position.xy, 0.0, 1.0) * u_modelView).xy; \n\
    vec2 len = vec2(a_length.x, a_length.z); \n\
    if (dot(a_normal, a_normal) != 0.0) \n\
    { \n\
      vec2 norm = a_normal * u_routeParams.x; \n\
      transformedAxisPos = calcLineTransformedAxisPos(transformedAxisPos, a_position.xy + norm, \n\
                                                      u_modelView, length(norm)); \n\
      if (u_routeParams.y != 0.0) \n\
        len = vec2(a_length.x + a_length.y * u_routeParams.y, a_length.z); \n\
    } \n\
    v_length = vec3(len, u_routeParams.z); \n\
    v_color = a_color; \n\
    vec4 pos = vec4(transformedAxisPos, a_position.z, 1.0) * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
  } \n\
";

char const MASKED_TEXTURING_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec4 a_position; \n\
  attribute vec2 a_normal; \n\
  attribute vec2 a_colorTexCoords; \n\
  attribute vec2 a_maskTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  varying vec2 v_colorTexCoords; \n\
  varying vec2 v_maskTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pos = vec4(a_position.xyz, 1) * u_modelView; \n\
    vec4 shiftedPos = vec4(a_normal, 0, 0) + pos; \n\
    gl_Position = applyPivotTransform(shiftedPos * u_projection, u_pivotTransform, 0.0); \n\
    v_colorTexCoords = a_colorTexCoords; \n\
    v_maskTexCoords = a_maskTexCoords; \n\
  } \n\
";

char const GLES3_MASKED_TEXTURING_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec4 a_position; \n\
  in vec2 a_normal; \n\
  in vec2 a_colorTexCoords; \n\
  in vec2 a_maskTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  out vec2 v_colorTexCoords; \n\
  out vec2 v_maskTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 pos = vec4(a_position.xyz, 1) * u_modelView; \n\
    vec4 shiftedPos = vec4(a_normal, 0, 0) + pos; \n\
    gl_Position = applyPivotTransform(shiftedPos * u_projection, u_pivotTransform, 0.0); \n\
    v_colorTexCoords = a_colorTexCoords; \n\
    v_maskTexCoords = a_maskTexCoords; \n\
  } \n\
";

char const CIRCLE_POINT_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec3 a_normal; \n\
  attribute vec3 a_position; \n\
  attribute vec4 a_color; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  varying vec3 v_radius; \n\
  varying vec4 v_color; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec3 radius = a_normal * a_position.z; \n\
    vec4 pos = vec4(a_position.xy, 0, 1) * u_modelView; \n\
    vec4 shiftedPos = vec4(radius.xy, 0, 0) + pos; \n\
    gl_Position = applyPivotTransform(shiftedPos * u_projection, u_pivotTransform, 0.0); \n\
    v_radius = radius; \n\
    v_color = a_color; \n\
  } \n\
";

char const GLES3_CIRCLE_POINT_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 a_normal; \n\
  in vec3 a_position; \n\
  in vec4 a_color; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  out vec3 v_radius; \n\
  out vec4 v_color; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec3 radius = a_normal * a_position.z; \n\
    vec4 pos = vec4(a_position.xy, 0, 1) * u_modelView; \n\
    vec4 shiftedPos = vec4(radius.xy, 0, 0) + pos; \n\
    gl_Position = applyPivotTransform(shiftedPos * u_projection, u_pivotTransform, 0.0); \n\
    v_radius = radius; \n\
    v_color = a_color; \n\
  } \n\
";

char const COLORED_SYMBOL_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec3 a_position; \n\
  attribute vec4 a_normal; \n\
  attribute vec4 a_colorTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  varying vec4 v_normal; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  varying LOW_P vec4 v_color; \n\
  #else \n\
  varying vec2 v_colorTexCoords; \n\
  #endif \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 p = vec4(a_position, 1) * u_modelView; \n\
    vec4 pos = vec4(a_normal.xy + a_colorTexCoords.zw, 0, 0) + p; \n\
    gl_Position = applyPivotTransform(pos * u_projection, u_pivotTransform, 0.0); \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture2D(u_colorTex, a_colorTexCoords.xy); \n\
  #else \n\
    v_colorTexCoords = a_colorTexCoords.xy; \n\
  #endif \n\
    v_normal = a_normal; \n\
  } \n\
";

char const GLES3_COLORED_SYMBOL_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 a_position; \n\
  in vec4 a_normal; \n\
  in vec4 a_colorTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  out vec4 v_normal; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  out LOW_P vec4 v_color; \n\
  #else \n\
  out vec2 v_colorTexCoords; \n\
  #endif \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 p = vec4(a_position, 1) * u_modelView; \n\
    vec4 pos = vec4(a_normal.xy + a_colorTexCoords.zw, 0, 0) + p; \n\
    gl_Position = applyPivotTransform(pos * u_projection, u_pivotTransform, 0.0); \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture(u_colorTex, a_colorTexCoords.xy); \n\
  #else \n\
    v_colorTexCoords = a_colorTexCoords.xy; \n\
  #endif \n\
    v_normal = a_normal; \n\
  } \n\
";

char const HATCHING_AREA_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform float u_opacity; \n\
  #ifdef ENABLE_VTF \n\
  varying LOW_P vec4 v_color; \n\
  #else \n\
  uniform sampler2D u_colorTex; \n\
  varying vec2 v_colorTexCoords; \n\
  #endif \n\
  uniform sampler2D u_maskTex; \n\
  varying vec2 v_maskTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
  #ifdef ENABLE_VTF \n\
    LOW_P vec4 color = v_color; \n\
  #else \n\
    LOW_P vec4 color = texture2D(u_colorTex, v_colorTexCoords); \n\
  #endif \n\
    color *= texture2D(u_maskTex, v_maskTexCoords); \n\
    color.a *= u_opacity; \n\
    gl_FragColor = color; \n\
  } \n\
";

char const GLES3_HATCHING_AREA_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  uniform float u_opacity; \n\
  #ifdef ENABLE_VTF \n\
  in LOW_P vec4 v_color; \n\
  #else \n\
  uniform sampler2D u_colorTex; \n\
  in vec2 v_colorTexCoords; \n\
  #endif \n\
  uniform sampler2D u_maskTex; \n\
  in vec2 v_maskTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
  #ifdef ENABLE_VTF \n\
    LOW_P vec4 color = v_color; \n\
  #else \n\
    LOW_P vec4 color = texture(u_colorTex, v_colorTexCoords); \n\
  #endif \n\
    color *= texture(u_maskTex, v_maskTexCoords); \n\
    color.a *= u_opacity; \n\
    v_FragColor = color; \n\
  } \n\
";

char const TEXTURING_GUI_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec2 a_position; \n\
  attribute vec2 a_colorTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  varying vec2 v_colorTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    gl_Position = vec4(a_position, 0, 1) * u_modelView * u_projection; \n\
  #ifdef VULKAN \n\
    gl_Position.y = -gl_Position.y; \n\
    gl_Position.z = (gl_Position.z  + gl_Position.w) * 0.5; \n\
  #endif \n\
    v_colorTexCoords = a_colorTexCoords; \n\
  } \n\
";

char const GLES3_TEXTURING_GUI_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec2 a_position; \n\
  in vec2 a_colorTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  out vec2 v_colorTexCoords; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    gl_Position = vec4(a_position, 0, 1) * u_modelView * u_projection; \n\
  #ifdef VULKAN \n\
    gl_Position.y = -gl_Position.y; \n\
    gl_Position.z = (gl_Position.z  + gl_Position.w) * 0.5; \n\
  #endif \n\
    v_colorTexCoords = a_colorTexCoords; \n\
  } \n\
";

char const SMAA_EDGES_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec2 a_pos; \n\
  attribute vec2 a_tcoord; \n\
  uniform vec4 u_framebufferMetrics; \n\
  varying vec2 v_colorTexCoords; \n\
  varying vec4 v_offset0; \n\
  varying vec4 v_offset1; \n\
  varying vec4 v_offset2; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    v_colorTexCoords = a_tcoord; \n\
    v_offset0 = u_framebufferMetrics.xyxy * vec4(-1.0, 0.0, 0.0, -1.0) + a_tcoord.xyxy; \n\
    v_offset1 = u_framebufferMetrics.xyxy * vec4( 1.0, 0.0, 0.0,  1.0) + a_tcoord.xyxy; \n\
    v_offset2 = u_framebufferMetrics.xyxy * vec4(-2.0, 0.0, 0.0, -2.0) + a_tcoord.xyxy; \n\
    gl_Position = vec4(a_pos, 0.0, 1.0); \n\
  } \n\
";

char const GLES3_SMAA_EDGES_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec2 a_pos; \n\
  in vec2 a_tcoord; \n\
  uniform vec4 u_framebufferMetrics; \n\
  out vec2 v_colorTexCoords; \n\
  out vec4 v_offset0; \n\
  out vec4 v_offset1; \n\
  out vec4 v_offset2; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    v_colorTexCoords = a_tcoord; \n\
    v_offset0 = u_framebufferMetrics.xyxy * vec4(-1.0, 0.0, 0.0, -1.0) + a_tcoord.xyxy; \n\
    v_offset1 = u_framebufferMetrics.xyxy * vec4( 1.0, 0.0, 0.0,  1.0) + a_tcoord.xyxy; \n\
    v_offset2 = u_framebufferMetrics.xyxy * vec4(-2.0, 0.0, 0.0, -2.0) + a_tcoord.xyxy; \n\
    gl_Position = vec4(a_pos, 0.0, 1.0); \n\
  } \n\
";

char const TRAFFIC_LINE_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec3 a_position; \n\
  attribute vec2 a_colorTexCoord; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  varying vec2 v_colorTexCoord; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec2 transformedAxisPos = (vec4(a_position.xy, 0.0, 1.0) * u_modelView).xy; \n\
    vec4 pos = vec4(transformedAxisPos, a_position.z, 1.0) * u_projection; \n\
    v_colorTexCoord = a_colorTexCoord; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
  } \n\
";

char const GLES3_TRAFFIC_LINE_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 a_position; \n\
  in vec2 a_colorTexCoord; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  out vec2 v_colorTexCoord; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec2 transformedAxisPos = (vec4(a_position.xy, 0.0, 1.0) * u_modelView).xy; \n\
    vec4 pos = vec4(transformedAxisPos, a_position.z, 1.0) * u_projection; \n\
    v_colorTexCoord = a_colorTexCoord; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
  } \n\
";

char const TRANSIT_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec3 a_position; \n\
  attribute vec4 a_normal; \n\
  attribute vec4 a_color; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_lineHalfWidth; \n\
  varying vec4 v_color; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec2 normal = a_normal.xy; \n\
    vec2 transformedAxisPos = (vec4(a_position.xy, 0.0, 1.0) * u_modelView).xy; \n\
    if (dot(normal, normal) != 0.0) \n\
    { \n\
      vec2 norm = normal * u_lineHalfWidth; \n\
      transformedAxisPos = calcLineTransformedAxisPos(transformedAxisPos, a_position.xy + norm, \n\
                                                      u_modelView, length(norm)); \n\
    } \n\
    v_color = a_color; \n\
    vec4 pos = vec4(transformedAxisPos, a_position.z, 1.0) * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
  } \n\
";

char const GLES3_TRANSIT_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 a_position; \n\
  in vec4 a_normal; \n\
  in vec4 a_color; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  uniform float u_lineHalfWidth; \n\
  out vec4 v_color; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec2 normal = a_normal.xy; \n\
    vec2 transformedAxisPos = (vec4(a_position.xy, 0.0, 1.0) * u_modelView).xy; \n\
    if (dot(normal, normal) != 0.0) \n\
    { \n\
      vec2 norm = normal * u_lineHalfWidth; \n\
      transformedAxisPos = calcLineTransformedAxisPos(transformedAxisPos, a_position.xy + norm, \n\
                                                      u_modelView, length(norm)); \n\
    } \n\
    v_color = a_color; \n\
    vec4 pos = vec4(transformedAxisPos, a_position.z, 1.0) * u_projection; \n\
    gl_Position = applyPivotTransform(pos, u_pivotTransform, 0.0); \n\
  } \n\
";

char const DASHED_LINE_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  varying vec2 v_colorTexCoord; \n\
  varying vec2 v_maskTexCoord; \n\
  uniform sampler2D u_colorTex; \n\
  uniform sampler2D u_maskTex; \n\
  uniform float u_opacity; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
    vec4 color = texture2D(u_colorTex, v_colorTexCoord); \n\
  #ifdef GLES3 \n\
    float mask = texture2D(u_maskTex, v_maskTexCoord).r; \n\
  #else \n\
    float mask = texture2D(u_maskTex, v_maskTexCoord).a; \n\
  #endif \n\
    color.a = color.a * mask * u_opacity; \n\
    gl_FragColor = color; \n\
  } \n\
";

char const GLES3_DASHED_LINE_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec2 v_colorTexCoord; \n\
  in vec2 v_maskTexCoord; \n\
  uniform sampler2D u_colorTex; \n\
  uniform sampler2D u_maskTex; \n\
  uniform float u_opacity; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
    vec4 color = texture(u_colorTex, v_colorTexCoord); \n\
  #ifdef GLES3 \n\
    float mask = texture(u_maskTex, v_maskTexCoord).r; \n\
  #else \n\
    float mask = texture(u_maskTex, v_maskTexCoord).a; \n\
  #endif \n\
    color.a = color.a * mask * u_opacity; \n\
    v_FragColor = color; \n\
  } \n\
";

char const TEXT_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  #ifdef ENABLE_VTF \n\
  varying LOW_P vec4 v_color; \n\
  #else \n\
  varying vec2 v_colorTexCoord; \n\
  uniform sampler2D u_colorTex; \n\
  #endif \n\
  varying vec2 v_maskTexCoord; \n\
  uniform sampler2D u_maskTex; \n\
  uniform float u_opacity; \n\
  uniform vec2 u_contrastGamma; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture2D(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  void main() \n\
  { \n\
  #ifdef ENABLE_VTF \n\
    LOW_P vec4 glyphColor = v_color; \n\
  #else \n\
    LOW_P vec4 glyphColor = texture2D(u_colorTex, v_colorTexCoord); \n\
  #endif \n\
  #ifdef GLES3 \n\
    float dist = texture2D(u_maskTex, v_maskTexCoord).r; \n\
  #else \n\
    float dist = texture2D(u_maskTex, v_maskTexCoord).a; \n\
  #endif \n\
    float alpha = smoothstep(u_contrastGamma.x - u_contrastGamma.y, u_contrastGamma.x + u_contrastGamma.y, dist) * u_opacity; \n\
    glyphColor.a *= alpha; \n\
    gl_FragColor = glyphColor; \n\
  } \n\
";

char const GLES3_TEXT_FSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  #ifdef ENABLE_VTF \n\
  in LOW_P vec4 v_color; \n\
  #else \n\
  in vec2 v_colorTexCoord; \n\
  uniform sampler2D u_colorTex; \n\
  #endif \n\
  in vec2 v_maskTexCoord; \n\
  uniform sampler2D u_maskTex; \n\
  uniform float u_opacity; \n\
  uniform vec2 u_contrastGamma; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 samsungGoogleNexusWorkaround(vec4 color) \n\
  { \n\
  #ifdef SAMSUNG_GOOGLE_NEXUS \n\
    const float kFakeColorScalar = 0.0; \n\
    return color + texture(u_colorTex, vec2(0.0, 0.0)) * kFakeColorScalar; \n\
  #else \n\
    return color; \n\
  #endif \n\
  } \n\
  out vec4 v_FragColor; \n\
  void main() \n\
  { \n\
  #ifdef ENABLE_VTF \n\
    LOW_P vec4 glyphColor = v_color; \n\
  #else \n\
    LOW_P vec4 glyphColor = texture(u_colorTex, v_colorTexCoord); \n\
  #endif \n\
  #ifdef GLES3 \n\
    float dist = texture(u_maskTex, v_maskTexCoord).r; \n\
  #else \n\
    float dist = texture(u_maskTex, v_maskTexCoord).a; \n\
  #endif \n\
    float alpha = smoothstep(u_contrastGamma.x - u_contrastGamma.y, u_contrastGamma.x + u_contrastGamma.y, dist) * u_opacity; \n\
    glyphColor.a *= alpha; \n\
    v_FragColor = glyphColor; \n\
  } \n\
";

char const CIRCLE_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec3 a_position; \n\
  attribute vec3 a_normal; \n\
  attribute vec2 a_colorTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  varying LOW_P vec4 v_color; \n\
  #else \n\
  varying vec2 v_colorTexCoords; \n\
  #endif \n\
  varying vec3 v_radius; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 p = vec4(a_position, 1) * u_modelView; \n\
    vec4 pos = vec4(a_normal.xy, 0, 0) + p; \n\
    gl_Position = applyPivotTransform(pos * u_projection, u_pivotTransform, 0.0); \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture2D(u_colorTex, a_colorTexCoords); \n\
  #else \n\
    v_colorTexCoords = a_colorTexCoords; \n\
  #endif \n\
    v_radius = a_normal; \n\
  } \n\
";

char const GLES3_CIRCLE_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 a_position; \n\
  in vec3 a_normal; \n\
  in vec2 a_colorTexCoords; \n\
  uniform mat4 u_modelView; \n\
  uniform mat4 u_projection; \n\
  uniform mat4 u_pivotTransform; \n\
  #ifdef ENABLE_VTF \n\
  uniform sampler2D u_colorTex; \n\
  out LOW_P vec4 v_color; \n\
  #else \n\
  out vec2 v_colorTexCoords; \n\
  #endif \n\
  out vec3 v_radius; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 p = vec4(a_position, 1) * u_modelView; \n\
    vec4 pos = vec4(a_normal.xy, 0, 0) + p; \n\
    gl_Position = applyPivotTransform(pos * u_projection, u_pivotTransform, 0.0); \n\
  #ifdef ENABLE_VTF \n\
    v_color = texture(u_colorTex, a_colorTexCoords); \n\
  #else \n\
    v_colorTexCoords = a_colorTexCoords; \n\
  #endif \n\
    v_radius = a_normal; \n\
  } \n\
";

char const SMAA_FINAL_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec2 a_pos; \n\
  attribute vec2 a_tcoord; \n\
  uniform vec4 u_framebufferMetrics; \n\
  varying vec2 v_colorTexCoords; \n\
  varying vec4 v_offset; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    v_colorTexCoords = a_tcoord; \n\
    v_offset = u_framebufferMetrics.xyxy * vec4(1.0, 0.0, 0.0, 1.0) + a_tcoord.xyxy; \n\
    gl_Position = vec4(a_pos, 0.0, 1.0); \n\
  } \n\
";

char const GLES3_SMAA_FINAL_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec2 a_pos; \n\
  in vec2 a_tcoord; \n\
  uniform vec4 u_framebufferMetrics; \n\
  out vec2 v_colorTexCoords; \n\
  out vec4 v_offset; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    v_colorTexCoords = a_tcoord; \n\
    v_offset = u_framebufferMetrics.xyxy * vec4(1.0, 0.0, 0.0, 1.0) + a_tcoord.xyxy; \n\
    gl_Position = vec4(a_pos, 0.0, 1.0); \n\
  } \n\
";

char const ARROW3D_SHADOW_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  attribute vec3 a_pos; \n\
  attribute vec2 a_texCoords; \n\
  uniform mat4 u_transform; \n\
  varying float v_intensity; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 position = u_transform * vec4(a_pos, 1.0); \n\
    v_intensity = a_texCoords.x; \n\
    gl_Position = position; \n\
  #ifdef VULKAN \n\
    gl_Position.y = -gl_Position.y; \n\
    gl_Position.z = (gl_Position.z + gl_Position.w) * 0.5; \n\
  #endif \n\
  } \n\
";

char const GLES3_ARROW3D_SHADOW_VSH[] = " \
  #ifdef GL_ES \n\
    #ifdef GL_FRAGMENT_PRECISION_HIGH \n\
      #define MAXPREC highp \n\
    #else \n\
      #define MAXPREC mediump \n\
    #endif \n\
    precision MAXPREC float; \n\
    #define LOW_P lowp \n\
    #define MEDIUM_P mediump \n\
    #define HIGH_P highp \n\
  #else \n\
    #define LOW_P \n\
    #define MEDIUM_P \n\
    #define HIGH_P \n\
  #endif \n\
  in vec3 a_pos; \n\
  in vec2 a_texCoords; \n\
  uniform mat4 u_transform; \n\
  out float v_intensity; \n\
  const float kShapeCoordScalar = 1000.0; \n\
  vec4 applyPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ) \n\
  { \n\
    vec4 transformedPivot = pivot; \n\
    float w = transformedPivot.w; \n\
    transformedPivot.xyw = (pivotTransform * vec4(transformedPivot.xy, pivotRealZ, w)).xyw; \n\
    transformedPivot.z *= transformedPivot.w / w; \n\
  #ifdef VULKAN \n\
    transformedPivot.y = -transformedPivot.y; \n\
    transformedPivot.z = (transformedPivot.z  + transformedPivot.w) / 2.0; \n\
  #endif \n\
    return transformedPivot; \n\
  } \n\
  vec4 applyBillboardPivotTransform(vec4 pivot, mat4 pivotTransform, float pivotRealZ, vec2 offset) \n\
  { \n\
    float logicZ = pivot.z / pivot.w; \n\
    vec4 transformedPivot = pivotTransform * vec4(pivot.xy, pivotRealZ, pivot.w); \n\
    vec4 scale = pivotTransform * vec4(1.0, -1.0, 0.0, 1.0); \n\
    vec4 position = vec4(transformedPivot.xy / transformedPivot.w, logicZ, 1.0) + vec4(offset / scale.w * scale.x, 0.0, 0.0); \n\
  #ifdef VULKAN \n\
    position.y = -position.y; \n\
    position.z = (position.z  + position.w) / 2.0; \n\
  #endif \n\
    return position; \n\
  } \n\
  vec2 calcLineTransformedAxisPos(vec2 originalAxisPos, vec2 shiftedPos, mat4 modelView, float halfWidth) \n\
  { \n\
    vec2 p = (vec4(shiftedPos, 0.0, 1.0) * modelView).xy; \n\
    vec2 d = p - originalAxisPos; \n\
    if (dot(d, d) != 0.0) \n\
      return originalAxisPos + normalize(d) * halfWidth; \n\
    else \n\
      return originalAxisPos; \n\
  } \n\
  void main() \n\
  { \n\
    vec4 position = u_transform * vec4(a_pos, 1.0); \n\
    v_intensity = a_texCoords.x; \n\
    gl_Position = position; \n\
  #ifdef VULKAN \n\
    gl_Position.y = -gl_Position.y; \n\
    gl_Position.z = (gl_Position.z + gl_Position.w) * 0.5; \n\
  #endif \n\
  } \n\
";

GLProgramInfo GetProgramInfo(dp::ApiVersion apiVersion, Program program)
{
  if (apiVersion == dp::ApiVersion::OpenGLES2)
  {
    static std::array<GLProgramInfo, static_cast<size_t>(Program::ProgramsCount)> gpuIndex = {{
      GLProgramInfo("COLORED_SYMBOL_VSH", "COLORED_SYMBOL_FSH", COLORED_SYMBOL_VSH, COLORED_SYMBOL_FSH),
      GLProgramInfo("TEXTURING_VSH", "TEXTURING_FSH", TEXTURING_VSH, TEXTURING_FSH),
      GLProgramInfo("MASKED_TEXTURING_VSH", "MASKED_TEXTURING_FSH", MASKED_TEXTURING_VSH, MASKED_TEXTURING_FSH),
      GLProgramInfo("USER_MARK_VSH", "USER_MARK_FSH", USER_MARK_VSH, USER_MARK_FSH),
      GLProgramInfo("USER_MARK_VSH", "USER_MARK_FSH", USER_MARK_VSH, USER_MARK_FSH),
      GLProgramInfo("TEXT_OUTLINED_VSH", "TEXT_FSH", TEXT_OUTLINED_VSH, TEXT_FSH),
      GLProgramInfo("TEXT_VSH", "TEXT_FSH", TEXT_VSH, TEXT_FSH),
      GLProgramInfo("TEXT_OUTLINED_GUI_VSH", "TEXT_FSH", TEXT_OUTLINED_GUI_VSH, TEXT_FSH),
      GLProgramInfo("TEXT_OUTLINED_GUI_VSH", "TEXT_FSH", TEXT_OUTLINED_GUI_VSH, TEXT_FSH),
      GLProgramInfo("AREA_VSH", "SOLID_COLOR_FSH", AREA_VSH, SOLID_COLOR_FSH),
      GLProgramInfo("AREA_VSH", "SOLID_COLOR_FSH", AREA_VSH, SOLID_COLOR_FSH),
      GLProgramInfo("AREA3D_VSH", "TEXTURING3D_FSH", AREA3D_VSH, TEXTURING3D_FSH),
      GLProgramInfo("AREA3D_OUTLINE_VSH", "SOLID_COLOR_FSH", AREA3D_OUTLINE_VSH, SOLID_COLOR_FSH),
      GLProgramInfo("LINE_VSH", "LINE_FSH", LINE_VSH, LINE_FSH),
      GLProgramInfo("TRANSIT_CIRCLE_VSH", "TRANSIT_CIRCLE_FSH", TRANSIT_CIRCLE_VSH, TRANSIT_CIRCLE_FSH),
      GLProgramInfo("DASHED_LINE_VSH", "DASHED_LINE_FSH", DASHED_LINE_VSH, DASHED_LINE_FSH),
      GLProgramInfo("PATH_SYMBOL_VSH", "TEXTURING_FSH", PATH_SYMBOL_VSH, TEXTURING_FSH),
      GLProgramInfo("AREA_VSH", "SOLID_COLOR_FSH", AREA_VSH, SOLID_COLOR_FSH),
      GLProgramInfo("CIRCLE_VSH", "CIRCLE_FSH", CIRCLE_VSH, CIRCLE_FSH),
      GLProgramInfo("HATCHING_AREA_VSH", "HATCHING_AREA_FSH", HATCHING_AREA_VSH, HATCHING_AREA_FSH),
      GLProgramInfo("TEXTURING_GUI_VSH", "TEXTURING_FSH", TEXTURING_GUI_VSH, TEXTURING_FSH),
      GLProgramInfo("RULER_VSH", "TEXTURING_FSH", RULER_VSH, TEXTURING_FSH),
      GLProgramInfo("POSITION_ACCURACY3D_VSH", "TEXTURING_FSH", POSITION_ACCURACY3D_VSH, TEXTURING_FSH),
      GLProgramInfo("MY_POSITION_VSH", "TEXTURING_FSH", MY_POSITION_VSH, TEXTURING_FSH),
      GLProgramInfo("SELECTION_LINE_VSH", "SELECTION_LINE_FSH", SELECTION_LINE_VSH, SELECTION_LINE_FSH),
      GLProgramInfo("TRANSIT_VSH", "TRANSIT_FSH", TRANSIT_VSH, TRANSIT_FSH),
      GLProgramInfo("TRANSIT_MARKER_VSH", "TRANSIT_MARKER_FSH", TRANSIT_MARKER_VSH, TRANSIT_MARKER_FSH),
      GLProgramInfo("ROUTE_VSH", "ROUTE_FSH", ROUTE_VSH, ROUTE_FSH),
      GLProgramInfo("ROUTE_VSH", "ROUTE_DASH_FSH", ROUTE_VSH, ROUTE_DASH_FSH),
      GLProgramInfo("ROUTE_ARROW_VSH", "ROUTE_ARROW_FSH", ROUTE_ARROW_VSH, ROUTE_ARROW_FSH),
      GLProgramInfo("ROUTE_MARKER_VSH", "ROUTE_MARKER_FSH", ROUTE_MARKER_VSH, ROUTE_MARKER_FSH),
      GLProgramInfo("CIRCLE_POINT_VSH", "CIRCLE_POINT_FSH", CIRCLE_POINT_VSH, CIRCLE_POINT_FSH),
      GLProgramInfo("USER_MARK_VSH", "USER_MARK_FSH", USER_MARK_VSH, USER_MARK_FSH),
      GLProgramInfo("USER_MARK_VSH", "USER_MARK_FSH", USER_MARK_VSH, USER_MARK_FSH),
      GLProgramInfo("DEBUG_RECT_VSH", "DEBUG_RECT_FSH", DEBUG_RECT_VSH, DEBUG_RECT_FSH),
      GLProgramInfo("SCREEN_QUAD_VSH", "TEXTURING_FSH", SCREEN_QUAD_VSH, TEXTURING_FSH),
      GLProgramInfo("ARROW3D_VSH", "ARROW3D_FSH", ARROW3D_VSH, ARROW3D_FSH),
      GLProgramInfo("ARROW3D_TEXTURED_VSH", "ARROW3D_TEXTURED_FSH", ARROW3D_TEXTURED_VSH, ARROW3D_TEXTURED_FSH),
      GLProgramInfo("ARROW3D_SHADOW_VSH", "ARROW3D_SHADOW_FSH", ARROW3D_SHADOW_VSH, ARROW3D_SHADOW_FSH),
      GLProgramInfo("ARROW3D_SHADOW_VSH", "ARROW3D_OUTLINE_FSH", ARROW3D_SHADOW_VSH, ARROW3D_OUTLINE_FSH),
      GLProgramInfo("COLORED_SYMBOL_BILLBOARD_VSH", "COLORED_SYMBOL_FSH", COLORED_SYMBOL_BILLBOARD_VSH, COLORED_SYMBOL_FSH),
      GLProgramInfo("TEXTURING_BILLBOARD_VSH", "TEXTURING_FSH", TEXTURING_BILLBOARD_VSH, TEXTURING_FSH),
      GLProgramInfo("MASKED_TEXTURING_BILLBOARD_VSH", "MASKED_TEXTURING_FSH", MASKED_TEXTURING_BILLBOARD_VSH, MASKED_TEXTURING_FSH),
      GLProgramInfo("USER_MARK_BILLBOARD_VSH", "USER_MARK_FSH", USER_MARK_BILLBOARD_VSH, USER_MARK_FSH),
      GLProgramInfo("USER_MARK_BILLBOARD_VSH", "USER_MARK_FSH", USER_MARK_BILLBOARD_VSH, USER_MARK_FSH),
      GLProgramInfo("USER_MARK_BILLBOARD_VSH", "USER_MARK_FSH", USER_MARK_BILLBOARD_VSH, USER_MARK_FSH),
      GLProgramInfo("USER_MARK_BILLBOARD_VSH", "USER_MARK_FSH", USER_MARK_BILLBOARD_VSH, USER_MARK_FSH),
      GLProgramInfo("TEXT_OUTLINED_BILLBOARD_VSH", "TEXT_FSH", TEXT_OUTLINED_BILLBOARD_VSH, TEXT_FSH),
      GLProgramInfo("TEXT_BILLBOARD_VSH", "TEXT_FSH", TEXT_BILLBOARD_VSH, TEXT_FSH),
      GLProgramInfo("TRAFFIC_VSH", "TRAFFIC_FSH", TRAFFIC_VSH, TRAFFIC_FSH),
      GLProgramInfo("TRAFFIC_LINE_VSH", "TRAFFIC_LINE_FSH", TRAFFIC_LINE_VSH, TRAFFIC_LINE_FSH),
      GLProgramInfo("TRAFFIC_CIRCLE_VSH", "TRAFFIC_CIRCLE_FSH", TRAFFIC_CIRCLE_VSH, TRAFFIC_CIRCLE_FSH),
      GLProgramInfo("SMAA_EDGES_VSH", "SMAA_EDGES_FSH", SMAA_EDGES_VSH, SMAA_EDGES_FSH),
      GLProgramInfo("SMAA_BLENDING_WEIGHT_VSH", "SMAA_BLENDING_WEIGHT_FSH", SMAA_BLENDING_WEIGHT_VSH, SMAA_BLENDING_WEIGHT_FSH),
      GLProgramInfo("SMAA_FINAL_VSH", "SMAA_FINAL_FSH", SMAA_FINAL_VSH, SMAA_FINAL_FSH),
    }};
    return gpuIndex[static_cast<size_t>(program)];
  }
  else if (apiVersion == dp::ApiVersion::OpenGLES3)
  {
    static std::array<GLProgramInfo, static_cast<size_t>(Program::ProgramsCount)> gpuIndex = {{
      GLProgramInfo("GLES3_COLORED_SYMBOL_VSH", "GLES3_COLORED_SYMBOL_FSH", GLES3_COLORED_SYMBOL_VSH, GLES3_COLORED_SYMBOL_FSH),
      GLProgramInfo("GLES3_TEXTURING_VSH", "GLES3_TEXTURING_FSH", GLES3_TEXTURING_VSH, GLES3_TEXTURING_FSH),
      GLProgramInfo("GLES3_MASKED_TEXTURING_VSH", "GLES3_MASKED_TEXTURING_FSH", GLES3_MASKED_TEXTURING_VSH, GLES3_MASKED_TEXTURING_FSH),
      GLProgramInfo("GLES3_USER_MARK_VSH", "GLES3_USER_MARK_FSH", GLES3_USER_MARK_VSH, GLES3_USER_MARK_FSH),
      GLProgramInfo("GLES3_USER_MARK_VSH", "GLES3_USER_MARK_FSH", GLES3_USER_MARK_VSH, GLES3_USER_MARK_FSH),
      GLProgramInfo("GLES3_TEXT_OUTLINED_VSH", "GLES3_TEXT_FSH", GLES3_TEXT_OUTLINED_VSH, GLES3_TEXT_FSH),
      GLProgramInfo("GLES3_TEXT_VSH", "GLES3_TEXT_FSH", GLES3_TEXT_VSH, GLES3_TEXT_FSH),
      GLProgramInfo("GLES3_TEXT_OUTLINED_GUI_VSH", "GLES3_TEXT_FSH", GLES3_TEXT_OUTLINED_GUI_VSH, GLES3_TEXT_FSH),
      GLProgramInfo("GLES3_TEXT_OUTLINED_GUI_VSH", "GLES3_TEXT_FSH", GLES3_TEXT_OUTLINED_GUI_VSH, GLES3_TEXT_FSH),
      GLProgramInfo("GLES3_AREA_VSH", "GLES3_SOLID_COLOR_FSH", GLES3_AREA_VSH, GLES3_SOLID_COLOR_FSH),
      GLProgramInfo("GLES3_AREA_VSH", "GLES3_SOLID_COLOR_FSH", GLES3_AREA_VSH, GLES3_SOLID_COLOR_FSH),
      GLProgramInfo("GLES3_AREA3D_VSH", "GLES3_TEXTURING3D_FSH", GLES3_AREA3D_VSH, GLES3_TEXTURING3D_FSH),
      GLProgramInfo("GLES3_AREA3D_OUTLINE_VSH", "GLES3_SOLID_COLOR_FSH", GLES3_AREA3D_OUTLINE_VSH, GLES3_SOLID_COLOR_FSH),
      GLProgramInfo("GLES3_LINE_VSH", "GLES3_LINE_FSH", GLES3_LINE_VSH, GLES3_LINE_FSH),
      GLProgramInfo("GLES3_TRANSIT_CIRCLE_VSH", "GLES3_TRANSIT_CIRCLE_FSH", GLES3_TRANSIT_CIRCLE_VSH, GLES3_TRANSIT_CIRCLE_FSH),
      GLProgramInfo("GLES3_DASHED_LINE_VSH", "GLES3_DASHED_LINE_FSH", GLES3_DASHED_LINE_VSH, GLES3_DASHED_LINE_FSH),
      GLProgramInfo("GLES3_PATH_SYMBOL_VSH", "GLES3_TEXTURING_FSH", GLES3_PATH_SYMBOL_VSH, GLES3_TEXTURING_FSH),
      GLProgramInfo("GLES3_AREA_VSH", "GLES3_SOLID_COLOR_FSH", GLES3_AREA_VSH, GLES3_SOLID_COLOR_FSH),
      GLProgramInfo("GLES3_CIRCLE_VSH", "GLES3_CIRCLE_FSH", GLES3_CIRCLE_VSH, GLES3_CIRCLE_FSH),
      GLProgramInfo("GLES3_HATCHING_AREA_VSH", "GLES3_HATCHING_AREA_FSH", GLES3_HATCHING_AREA_VSH, GLES3_HATCHING_AREA_FSH),
      GLProgramInfo("GLES3_TEXTURING_GUI_VSH", "GLES3_TEXTURING_FSH", GLES3_TEXTURING_GUI_VSH, GLES3_TEXTURING_FSH),
      GLProgramInfo("GLES3_RULER_VSH", "GLES3_TEXTURING_FSH", GLES3_RULER_VSH, GLES3_TEXTURING_FSH),
      GLProgramInfo("GLES3_POSITION_ACCURACY3D_VSH", "GLES3_TEXTURING_FSH", GLES3_POSITION_ACCURACY3D_VSH, GLES3_TEXTURING_FSH),
      GLProgramInfo("GLES3_MY_POSITION_VSH", "GLES3_TEXTURING_FSH", GLES3_MY_POSITION_VSH, GLES3_TEXTURING_FSH),
      GLProgramInfo("GLES3_SELECTION_LINE_VSH", "GLES3_SELECTION_LINE_FSH", GLES3_SELECTION_LINE_VSH, GLES3_SELECTION_LINE_FSH),
      GLProgramInfo("GLES3_TRANSIT_VSH", "GLES3_TRANSIT_FSH", GLES3_TRANSIT_VSH, GLES3_TRANSIT_FSH),
      GLProgramInfo("GLES3_TRANSIT_MARKER_VSH", "GLES3_TRANSIT_MARKER_FSH", GLES3_TRANSIT_MARKER_VSH, GLES3_TRANSIT_MARKER_FSH),
      GLProgramInfo("GLES3_ROUTE_VSH", "GLES3_ROUTE_FSH", GLES3_ROUTE_VSH, GLES3_ROUTE_FSH),
      GLProgramInfo("GLES3_ROUTE_VSH", "GLES3_ROUTE_DASH_FSH", GLES3_ROUTE_VSH, GLES3_ROUTE_DASH_FSH),
      GLProgramInfo("GLES3_ROUTE_ARROW_VSH", "GLES3_ROUTE_ARROW_FSH", GLES3_ROUTE_ARROW_VSH, GLES3_ROUTE_ARROW_FSH),
      GLProgramInfo("GLES3_ROUTE_MARKER_VSH", "GLES3_ROUTE_MARKER_FSH", GLES3_ROUTE_MARKER_VSH, GLES3_ROUTE_MARKER_FSH),
      GLProgramInfo("GLES3_CIRCLE_POINT_VSH", "GLES3_CIRCLE_POINT_FSH", GLES3_CIRCLE_POINT_VSH, GLES3_CIRCLE_POINT_FSH),
      GLProgramInfo("GLES3_USER_MARK_VSH", "GLES3_USER_MARK_FSH", GLES3_USER_MARK_VSH, GLES3_USER_MARK_FSH),
      GLProgramInfo("GLES3_USER_MARK_VSH", "GLES3_USER_MARK_FSH", GLES3_USER_MARK_VSH, GLES3_USER_MARK_FSH),
      GLProgramInfo("GLES3_DEBUG_RECT_VSH", "GLES3_DEBUG_RECT_FSH", GLES3_DEBUG_RECT_VSH, GLES3_DEBUG_RECT_FSH),
      GLProgramInfo("GLES3_SCREEN_QUAD_VSH", "GLES3_TEXTURING_FSH", GLES3_SCREEN_QUAD_VSH, GLES3_TEXTURING_FSH),
      GLProgramInfo("GLES3_ARROW3D_VSH", "GLES3_ARROW3D_FSH", GLES3_ARROW3D_VSH, GLES3_ARROW3D_FSH),
      GLProgramInfo("GLES3_ARROW3D_TEXTURED_VSH", "GLES3_ARROW3D_TEXTURED_FSH", GLES3_ARROW3D_TEXTURED_VSH, GLES3_ARROW3D_TEXTURED_FSH),
      GLProgramInfo("GLES3_ARROW3D_SHADOW_VSH", "GLES3_ARROW3D_SHADOW_FSH", GLES3_ARROW3D_SHADOW_VSH, GLES3_ARROW3D_SHADOW_FSH),
      GLProgramInfo("GLES3_ARROW3D_SHADOW_VSH", "GLES3_ARROW3D_OUTLINE_FSH", GLES3_ARROW3D_SHADOW_VSH, GLES3_ARROW3D_OUTLINE_FSH),
      GLProgramInfo("GLES3_COLORED_SYMBOL_BILLBOARD_VSH", "GLES3_COLORED_SYMBOL_FSH", GLES3_COLORED_SYMBOL_BILLBOARD_VSH, GLES3_COLORED_SYMBOL_FSH),
      GLProgramInfo("GLES3_TEXTURING_BILLBOARD_VSH", "GLES3_TEXTURING_FSH", GLES3_TEXTURING_BILLBOARD_VSH, GLES3_TEXTURING_FSH),
      GLProgramInfo("GLES3_MASKED_TEXTURING_BILLBOARD_VSH", "GLES3_MASKED_TEXTURING_FSH", GLES3_MASKED_TEXTURING_BILLBOARD_VSH, GLES3_MASKED_TEXTURING_FSH),
      GLProgramInfo("GLES3_USER_MARK_BILLBOARD_VSH", "GLES3_USER_MARK_FSH", GLES3_USER_MARK_BILLBOARD_VSH, GLES3_USER_MARK_FSH),
      GLProgramInfo("GLES3_USER_MARK_BILLBOARD_VSH", "GLES3_USER_MARK_FSH", GLES3_USER_MARK_BILLBOARD_VSH, GLES3_USER_MARK_FSH),
      GLProgramInfo("GLES3_USER_MARK_BILLBOARD_VSH", "GLES3_USER_MARK_FSH", GLES3_USER_MARK_BILLBOARD_VSH, GLES3_USER_MARK_FSH),
      GLProgramInfo("GLES3_USER_MARK_BILLBOARD_VSH", "GLES3_USER_MARK_FSH", GLES3_USER_MARK_BILLBOARD_VSH, GLES3_USER_MARK_FSH),
      GLProgramInfo("GLES3_TEXT_OUTLINED_BILLBOARD_VSH", "GLES3_TEXT_FSH", GLES3_TEXT_OUTLINED_BILLBOARD_VSH, GLES3_TEXT_FSH),
      GLProgramInfo("GLES3_TEXT_BILLBOARD_VSH", "GLES3_TEXT_FSH", GLES3_TEXT_BILLBOARD_VSH, GLES3_TEXT_FSH),
      GLProgramInfo("GLES3_TRAFFIC_VSH", "GLES3_TRAFFIC_FSH", GLES3_TRAFFIC_VSH, GLES3_TRAFFIC_FSH),
      GLProgramInfo("GLES3_TRAFFIC_LINE_VSH", "GLES3_TRAFFIC_LINE_FSH", GLES3_TRAFFIC_LINE_VSH, GLES3_TRAFFIC_LINE_FSH),
      GLProgramInfo("GLES3_TRAFFIC_CIRCLE_VSH", "GLES3_TRAFFIC_CIRCLE_FSH", GLES3_TRAFFIC_CIRCLE_VSH, GLES3_TRAFFIC_CIRCLE_FSH),
      GLProgramInfo("GLES3_SMAA_EDGES_VSH", "GLES3_SMAA_EDGES_FSH", GLES3_SMAA_EDGES_VSH, GLES3_SMAA_EDGES_FSH),
      GLProgramInfo("GLES3_SMAA_BLENDING_WEIGHT_VSH", "GLES3_SMAA_BLENDING_WEIGHT_FSH", GLES3_SMAA_BLENDING_WEIGHT_VSH, GLES3_SMAA_BLENDING_WEIGHT_FSH),
      GLProgramInfo("GLES3_SMAA_FINAL_VSH", "GLES3_SMAA_FINAL_FSH", GLES3_SMAA_FINAL_VSH, GLES3_SMAA_FINAL_FSH),
    }};
    return gpuIndex[static_cast<size_t>(program)];
  }
  CHECK(false, ("Unsupported API version."));
  return {};
}
}  // namespace gpu
